# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Cluster(pulumi.CustomResource):
    additional_zones: pulumi.Output[list]
    """
    The list of zones in which the cluster's nodes
    should be located. These must be in the same region as the cluster zone for
    zonal clusters, or in the region of a regional cluster. In a multi-zonal cluster,
    the number of nodes specified in `initial_node_count` is created in
    all specified zones as well as the primary zone. If specified for a regional
    cluster, nodes will only be created in these zones. `additional_zones` has been
    deprecated in favour of `node_locations`.
    """
    addons_config: pulumi.Output[dict]
    """
    The configuration for addons supported by GKE.
    Structure is documented below.
    
      * `cloudrunConfig` (`dict`)
    
        * `disabled` (`bool`)
    
      * `horizontalPodAutoscaling` (`dict`)
    
        * `disabled` (`bool`)
    
      * `httpLoadBalancing` (`dict`)
    
        * `disabled` (`bool`)
    
      * `istioConfig` (`dict`)
    
        * `auth` (`str`)
        * `disabled` (`bool`)
    
      * `kubernetesDashboard` (`dict`)
    
        * `disabled` (`bool`)
    
      * `networkPolicyConfig` (`dict`)
    
        * `disabled` (`bool`)
    """
    authenticator_groups_config: pulumi.Output[dict]
    """
    ) Configuration for the
    [Google Groups for GKE](https://cloud.google.com/kubernetes-engine/docs/how-to/role-based-access-control#groups-setup-gsuite) feature.
    Structure is documented below.
    
      * `securityGroup` (`str`)
    """
    cluster_autoscaling: pulumi.Output[dict]
    """
    )
    Per-cluster configuration of Node Auto-Provisioning with Cluster Autoscaler to
    automatically adjust the size of the cluster and create/delete node pools based
    on the current needs of the cluster's workload. See the
    [guide to using Node Auto-Provisioning](https://cloud.google.com/kubernetes-engine/docs/how-to/node-auto-provisioning)
    for more details. Structure is documented below.
    
      * `enabled` (`bool`)
      * `resourceLimits` (`list`)
    
        * `maximum` (`float`)
        * `minimum` (`float`)
        * `resourceType` (`str`)
    """
    cluster_ipv4_cidr: pulumi.Output[str]
    """
    The IP address range of the Kubernetes pods
    in this cluster in CIDR notation (e.g. 10.96.0.0/14). Leave blank to have one
    automatically chosen or specify a /14 block in 10.0.0.0/8. This field will only
    work if your cluster is not VPC-native- when an `ip_allocation_policy` block is
    not defined, or `ip_allocation_policy.use_ip_aliases` is set to false. If your
    cluster is VPC-native, use `ip_allocation_policy.cluster_ipv4_cidr_block`.
    """
    database_encryption: pulumi.Output[dict]
    """
    ).
    Structure is documented below.
    
      * `keyName` (`str`)
      * `state` (`str`)
    """
    default_max_pods_per_node: pulumi.Output[float]
    """
    The default maximum number of pods
    per node in this cluster. This doesn't work on "routes-based" clusters, clusters
    that don't have IP Aliasing enabled. See the [official documentation](https://cloud.google.com/kubernetes-engine/docs/how-to/flexible-pod-cidr)
    for more information.
    """
    description: pulumi.Output[str]
    """
    Description of the cluster.
    """
    enable_binary_authorization: pulumi.Output[bool]
    """
    ) Enable Binary Authorization for this cluster.
    If enabled, all container images will be validated by Google Binary Authorization.
    """
    enable_intranode_visibility: pulumi.Output[bool]
    """
    )
    Whether Intra-node visibility is enabled for this cluster. This makes same node pod to pod traffic visible for VPC network.
    """
    enable_kubernetes_alpha: pulumi.Output[bool]
    """
    Whether to enable Kubernetes Alpha features for
    this cluster. Note that when this option is enabled, the cluster cannot be upgraded
    and will be automatically deleted after 30 days.
    """
    enable_legacy_abac: pulumi.Output[bool]
    """
    Whether the ABAC authorizer is enabled for this cluster.
    When enabled, identities in the system, including service accounts, nodes, and controllers,
    will have statically granted permissions beyond those provided by the RBAC configuration or IAM.
    Defaults to `false`
    """
    enable_shielded_nodes: pulumi.Output[bool]
    """
    ) Enable Shielded Nodes features on all nodes in this cluster.  Defaults to `false`.
    """
    enable_tpu: pulumi.Output[bool]
    """
    ) Whether to enable Cloud TPU resources in this cluster.
    See the [official documentation](https://cloud.google.com/tpu/docs/kubernetes-engine-setup).
    """
    endpoint: pulumi.Output[str]
    """
    The IP address of this cluster's Kubernetes master.
    """
    initial_node_count: pulumi.Output[float]
    """
    The number of nodes to create in this
    cluster's default node pool. In regional or multi-zonal clusters, this is the
    number of nodes per zone. Must be set if `node_pool` is not set. If you're using
    `container.NodePool` objects with no default node pool, you'll need to
    set this to a value of at least `1`, alongside setting
    `remove_default_node_pool` to `true`.
    """
    instance_group_urls: pulumi.Output[list]
    """
    List of instance group URLs which have been assigned
    to the cluster.
    """
    ip_allocation_policy: pulumi.Output[dict]
    """
    Configuration for cluster IP allocation. As of now, only pre-allocated subnetworks (custom type with secondary ranges) are supported.
    This will activate IP aliases. See the [official documentation](https://cloud.google.com/kubernetes-engine/docs/how-to/ip-aliases)
    Structure is documented below. This field is marked to use [Attribute as Block](https://www.terraform.io/docs/configuration/attr-as-blocks.html)
    in order to support explicit removal with `ip_allocation_policy = []`.
    
      * `clusterIpv4CidrBlock` (`str`)
      * `clusterSecondaryRangeName` (`str`)
      * `createSubnetwork` (`bool`)
      * `nodeIpv4CidrBlock` (`str`)
      * `servicesIpv4CidrBlock` (`str`)
      * `servicesSecondaryRangeName` (`str`)
      * `subnetworkName` (`str`)
      * `useIpAliases` (`bool`)
    """
    location: pulumi.Output[str]
    """
    The location (region or zone) in which the cluster
    master will be created, as well as the default node location. If you specify a
    zone (such as `us-central1-a`), the cluster will be a zonal cluster with a
    single cluster master. If you specify a region (such as `us-west1`), the
    cluster will be a regional cluster with multiple masters spread across zones in
    the region, and with default node locations in those zones as well.
    """
    logging_service: pulumi.Output[str]
    """
    The logging service that the cluster should
    write logs to. Available options include `logging.googleapis.com`,
    `logging.googleapis.com/kubernetes`, and `none`. Defaults to `logging.googleapis.com`
    """
    maintenance_policy: pulumi.Output[dict]
    """
    The maintenance policy to use for the cluster. Structure is
    documented below.
    
      * `dailyMaintenanceWindow` (`dict`)
    
        * `duration` (`str`)
        * `startTime` (`str`)
    """
    master_auth: pulumi.Output[dict]
    """
    The authentication information for accessing the
    Kubernetes master. Some values in this block are only returned by the API if
    your service account has permission to get credentials for your GKE cluster. If
    you see an unexpected diff removing a username/password or unsetting your client
    cert, ensure you have the `container.clusters.getCredentials` permission.
    Structure is documented below.
    
      * `clientCertificate` (`str`)
      * `clientCertificateConfig` (`dict`)
    
        * `issueClientCertificate` (`bool`)
    
      * `clientKey` (`str`)
      * `clusterCaCertificate` (`str`)
      * `password` (`str`)
      * `username` (`str`)
    """
    master_authorized_networks_config: pulumi.Output[dict]
    """
    The desired configuration options
    for master authorized networks. Omit the nested `cidr_blocks` attribute to disallow
    external access (except the cluster node IPs, which GKE automatically whitelists).
    
      * `cidrBlocks` (`list`)
    
        * `cidrBlock` (`str`)
        * `display_name` (`str`)
    """
    master_version: pulumi.Output[str]
    """
    The current version of the master in the cluster. This may
    be different than the `min_master_version` set in the config if the master
    has been updated by GKE.
    """
    min_master_version: pulumi.Output[str]
    """
    The minimum version of the master. GKE
    will auto-update the master to new versions, so this does not guarantee the
    current master version--use the read-only `master_version` field to obtain that.
    If unset, the cluster's version will be set by GKE to the version of the most recent
    official release (which is not necessarily the latest version).  Most users will find
    the `container.getEngineVersions` data source useful - it indicates which versions
    are available, and can be use to approximate fuzzy versions. If you intend to specify versions manually,
    [the docs](https://cloud.google.com/kubernetes-engine/versioning-and-upgrades#specifying_cluster_version)
    describe the various acceptable formats for this field.
    """
    monitoring_service: pulumi.Output[str]
    """
    The monitoring service that the cluster
    should write metrics to.
    Automatically send metrics from pods in the cluster to the Google Cloud Monitoring API.
    VM metrics will be collected by Google Compute Engine regardless of this setting
    Available options include
    `monitoring.googleapis.com`, `monitoring.googleapis.com/kubernetes`, and `none`.
    Defaults to `monitoring.googleapis.com`
    """
    name: pulumi.Output[str]
    """
    The name of the cluster, unique within the project and
    location.
    """
    network: pulumi.Output[str]
    """
    The name or self_link of the Google Compute Engine
    network to which the cluster is connected. For Shared VPC, set this to the self link of the
    shared network.
    """
    network_policy: pulumi.Output[dict]
    """
    Configuration options for the
    [NetworkPolicy](https://kubernetes.io/docs/concepts/services-networking/networkpolicies/)
    feature. Structure is documented below.
    
      * `enabled` (`bool`)
      * `provider` (`str`)
    """
    node_config: pulumi.Output[dict]
    """
    Parameters used in creating the default node pool.
    Generally, this field should not be used at the same time as a
    `container.NodePool` or a `node_pool` block; this configuration
    manages the default node pool, which isn't recommended to be used with
    this provider. Structure is documented below.
    
      * `disk_size_gb` (`float`)
      * `diskType` (`str`)
      * `guest_accelerators` (`list`)
    
        * `count` (`float`)
        * `type` (`str`)
    
      * `imageType` (`str`)
      * `labels` (`dict`)
      * `localSsdCount` (`float`)
      * `machine_type` (`str`)
      * `metadata` (`dict`)
      * `min_cpu_platform` (`str`)
      * `oauthScopes` (`list`)
      * `preemptible` (`bool`)
      * `sandboxConfig` (`dict`)
    
        * `sandboxType` (`str`)
    
      * `service_account` (`str`)
      * `shielded_instance_config` (`dict`)
    
        * `enableIntegrityMonitoring` (`bool`)
        * `enableSecureBoot` (`bool`)
    
      * `tags` (`list`)
      * `taints` (`list`)
    
        * `effect` (`str`)
        * `key` (`str`)
        * `value` (`str`)
    
      * `workloadMetadataConfig` (`dict`)
    
        * `nodeMetadata` (`str`)
    """
    node_locations: pulumi.Output[list]
    """
    The list of zones in which the cluster's nodes
    are located. Nodes must be in the region of their regional cluster or in the
    same region as their cluster's zone for zonal clusters. If this is specified for
    a zonal cluster, omit the cluster's zone.
    """
    node_pools: pulumi.Output[list]
    """
    List of node pools associated with this cluster.
    See container.NodePool for schema.
    **Warning:** node pools defined inside a cluster can't be changed (or added/removed) after
    cluster creation without deleting and recreating the entire cluster. Unless you absolutely need the ability
    to say "these are the _only_ node pools associated with this cluster", use the
    container.NodePool resource instead of this property.
    
      * `autoscaling` (`dict`)
    
        * `maxNodeCount` (`float`)
        * `minNodeCount` (`float`)
    
      * `initial_node_count` (`float`) - The number of nodes to create in this
        cluster's default node pool. In regional or multi-zonal clusters, this is the
        number of nodes per zone. Must be set if `node_pool` is not set. If you're using
        `container.NodePool` objects with no default node pool, you'll need to
        set this to a value of at least `1`, alongside setting
        `remove_default_node_pool` to `true`.
      * `instance_group_urls` (`list`) - List of instance group URLs which have been assigned
        to the cluster.
      * `management` (`dict`)
    
        * `autoRepair` (`bool`)
        * `autoUpgrade` (`bool`)
    
      * `max_pods_per_node` (`float`)
      * `name` (`str`) - The name of the cluster, unique within the project and
        location.
      * `name_prefix` (`str`)
      * `node_config` (`dict`) - Parameters used in creating the default node pool.
        Generally, this field should not be used at the same time as a
        `container.NodePool` or a `node_pool` block; this configuration
        manages the default node pool, which isn't recommended to be used with
        this provider. Structure is documented below.
    
        * `disk_size_gb` (`float`)
        * `diskType` (`str`)
        * `guest_accelerators` (`list`)
    
          * `count` (`float`)
          * `type` (`str`)
    
        * `imageType` (`str`)
        * `labels` (`dict`)
        * `localSsdCount` (`float`)
        * `machine_type` (`str`)
        * `metadata` (`dict`)
        * `min_cpu_platform` (`str`)
        * `oauthScopes` (`list`)
        * `preemptible` (`bool`)
        * `sandboxConfig` (`dict`)
    
          * `sandboxType` (`str`)
    
        * `service_account` (`str`)
        * `shielded_instance_config` (`dict`)
    
          * `enableIntegrityMonitoring` (`bool`)
          * `enableSecureBoot` (`bool`)
    
        * `tags` (`list`)
        * `taints` (`list`)
    
          * `effect` (`str`)
          * `key` (`str`)
          * `value` (`str`)
    
        * `workloadMetadataConfig` (`dict`)
    
          * `nodeMetadata` (`str`)
    
      * `node_count` (`float`)
      * `node_locations` (`list`) - The list of zones in which the cluster's nodes
        are located. Nodes must be in the region of their regional cluster or in the
        same region as their cluster's zone for zonal clusters. If this is specified for
        a zonal cluster, omit the cluster's zone.
      * `version` (`str`)
    """
    node_version: pulumi.Output[str]
    """
    The Kubernetes version on the nodes. Must either be unset
    or set to the same value as `min_master_version` on create. Defaults to the default
    version set by GKE which is not necessarily the latest version. This only affects
    nodes in the default node pool. While a fuzzy version can be specified, it's
    recommended that you specify explicit versions as this provider will see spurious diffs
    when fuzzy versions are used. See the `container.getEngineVersions` data source's
    `version_prefix` field to approximate fuzzy versions.
    To update nodes in other node pools, use the `version` attribute on the node pool.
    """
    pod_security_policy_config: pulumi.Output[dict]
    """
    ) Configuration for the
    [PodSecurityPolicy](https://cloud.google.com/kubernetes-engine/docs/how-to/pod-security-policies) feature.
    Structure is documented below.
    
      * `enabled` (`bool`)
    """
    private_cluster_config: pulumi.Output[dict]
    """
    A set of options for creating
    a private cluster. Structure is documented below.
    
      * `enablePrivateEndpoint` (`bool`)
      * `enablePrivateNodes` (`bool`)
      * `masterIpv4CidrBlock` (`str`)
      * `privateEndpoint` (`str`)
      * `publicEndpoint` (`str`)
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs. If it
    is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    release_channel: pulumi.Output[dict]
    """
    ) Configuration options for the
    [Release channel](https://cloud.google.com/kubernetes-engine/docs/concepts/release-channels)
    feature, which provide more control over automatic upgrades of your GKE clusters. Structure is documented below.
    
      * `channel` (`str`)
    """
    remove_default_node_pool: pulumi.Output[bool]
    """
    If `true`, deletes the default node
    pool upon cluster creation. If you're using `container.NodePool`
    resources with no default node pool, this should be set to `true`, alongside
    setting `initial_node_count` to at least `1`.
    """
    resource_labels: pulumi.Output[dict]
    """
    The GCE resource labels (a map of key/value pairs) to be applied to the cluster.
    """
    resource_usage_export_config: pulumi.Output[dict]
    """
    ) Configuration for the
    [ResourceUsageExportConfig](https://cloud.google.com/kubernetes-engine/docs/how-to/cluster-usage-metering) feature.
    Structure is documented below.
    
      * `bigqueryDestination` (`dict`)
    
        * `dataset_id` (`str`)
    
      * `enableNetworkEgressMetering` (`bool`)
    """
    services_ipv4_cidr: pulumi.Output[str]
    """
    The IP address range of the Kubernetes services in this
    cluster, in [CIDR](http://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing)
    notation (e.g. `1.2.3.4/29`). Service addresses are typically put in the last
    `/16` from the container CIDR.
    """
    subnetwork: pulumi.Output[str]
    """
    The name or self_link of the Google Compute Engine subnetwork in
    which the cluster's instances are launched.
    """
    tpu_ipv4_cidr_block: pulumi.Output[str]
    vertical_pod_autoscaling: pulumi.Output[dict]
    """
    )
    Vertical Pod Autoscaling automatically adjusts the resources of pods controlled by it.
    Structure is documented below.
    
      * `enabled` (`bool`)
    """
    workload_identity_config: pulumi.Output[dict]
    """
    )
    Workload Identity allows Kubernetes service accounts to act as a user-managed
    [Google IAM Service Account](https://cloud.google.com/iam/docs/service-accounts#user-managed_service_accounts).
    Structure is documented below.
    
      * `identityNamespace` (`str`)
    """
    zone: pulumi.Output[str]
    """
    The zone that the cluster master and nodes
    should be created in. If specified, this cluster will be a zonal cluster. `zone`
    has been deprecated in favour of `location`.
    """
    def __init__(__self__, resource_name, opts=None, additional_zones=None, addons_config=None, authenticator_groups_config=None, cluster_autoscaling=None, cluster_ipv4_cidr=None, database_encryption=None, default_max_pods_per_node=None, description=None, enable_binary_authorization=None, enable_intranode_visibility=None, enable_kubernetes_alpha=None, enable_legacy_abac=None, enable_shielded_nodes=None, enable_tpu=None, initial_node_count=None, ip_allocation_policy=None, location=None, logging_service=None, maintenance_policy=None, master_auth=None, master_authorized_networks_config=None, min_master_version=None, monitoring_service=None, name=None, network=None, network_policy=None, node_config=None, node_locations=None, node_pools=None, node_version=None, pod_security_policy_config=None, private_cluster_config=None, project=None, region=None, release_channel=None, remove_default_node_pool=None, resource_labels=None, resource_usage_export_config=None, subnetwork=None, vertical_pod_autoscaling=None, workload_identity_config=None, zone=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a Google Kubernetes Engine (GKE) cluster. For more information see
        [the official documentation](https://cloud.google.com/container-engine/docs/clusters)
        and [the API reference](https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.locations.clusters).
        
        > **Note:** All arguments and attributes, including basic auth username and
        passwords as well as certificate outputs will be stored in the raw state as
        plaintext. [Read more about sensitive data in state](https://www.terraform.io/docs/state/sensitive-data.html).
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] additional_zones: The list of zones in which the cluster's nodes
               should be located. These must be in the same region as the cluster zone for
               zonal clusters, or in the region of a regional cluster. In a multi-zonal cluster,
               the number of nodes specified in `initial_node_count` is created in
               all specified zones as well as the primary zone. If specified for a regional
               cluster, nodes will only be created in these zones. `additional_zones` has been
               deprecated in favour of `node_locations`.
        :param pulumi.Input[dict] addons_config: The configuration for addons supported by GKE.
               Structure is documented below.
        :param pulumi.Input[dict] authenticator_groups_config: ) Configuration for the
               [Google Groups for GKE](https://cloud.google.com/kubernetes-engine/docs/how-to/role-based-access-control#groups-setup-gsuite) feature.
               Structure is documented below.
        :param pulumi.Input[dict] cluster_autoscaling: )
               Per-cluster configuration of Node Auto-Provisioning with Cluster Autoscaler to
               automatically adjust the size of the cluster and create/delete node pools based
               on the current needs of the cluster's workload. See the
               [guide to using Node Auto-Provisioning](https://cloud.google.com/kubernetes-engine/docs/how-to/node-auto-provisioning)
               for more details. Structure is documented below.
        :param pulumi.Input[str] cluster_ipv4_cidr: The IP address range of the Kubernetes pods
               in this cluster in CIDR notation (e.g. 10.96.0.0/14). Leave blank to have one
               automatically chosen or specify a /14 block in 10.0.0.0/8. This field will only
               work if your cluster is not VPC-native- when an `ip_allocation_policy` block is
               not defined, or `ip_allocation_policy.use_ip_aliases` is set to false. If your
               cluster is VPC-native, use `ip_allocation_policy.cluster_ipv4_cidr_block`.
        :param pulumi.Input[dict] database_encryption: ).
               Structure is documented below.
        :param pulumi.Input[float] default_max_pods_per_node: The default maximum number of pods
               per node in this cluster. This doesn't work on "routes-based" clusters, clusters
               that don't have IP Aliasing enabled. See the [official documentation](https://cloud.google.com/kubernetes-engine/docs/how-to/flexible-pod-cidr)
               for more information.
        :param pulumi.Input[str] description: Description of the cluster.
        :param pulumi.Input[bool] enable_binary_authorization: ) Enable Binary Authorization for this cluster.
               If enabled, all container images will be validated by Google Binary Authorization.
        :param pulumi.Input[bool] enable_intranode_visibility: )
               Whether Intra-node visibility is enabled for this cluster. This makes same node pod to pod traffic visible for VPC network.
        :param pulumi.Input[bool] enable_kubernetes_alpha: Whether to enable Kubernetes Alpha features for
               this cluster. Note that when this option is enabled, the cluster cannot be upgraded
               and will be automatically deleted after 30 days.
        :param pulumi.Input[bool] enable_legacy_abac: Whether the ABAC authorizer is enabled for this cluster.
               When enabled, identities in the system, including service accounts, nodes, and controllers,
               will have statically granted permissions beyond those provided by the RBAC configuration or IAM.
               Defaults to `false`
        :param pulumi.Input[bool] enable_shielded_nodes: ) Enable Shielded Nodes features on all nodes in this cluster.  Defaults to `false`.
        :param pulumi.Input[bool] enable_tpu: ) Whether to enable Cloud TPU resources in this cluster.
               See the [official documentation](https://cloud.google.com/tpu/docs/kubernetes-engine-setup).
        :param pulumi.Input[float] initial_node_count: The number of nodes to create in this
               cluster's default node pool. In regional or multi-zonal clusters, this is the
               number of nodes per zone. Must be set if `node_pool` is not set. If you're using
               `container.NodePool` objects with no default node pool, you'll need to
               set this to a value of at least `1`, alongside setting
               `remove_default_node_pool` to `true`.
        :param pulumi.Input[dict] ip_allocation_policy: Configuration for cluster IP allocation. As of now, only pre-allocated subnetworks (custom type with secondary ranges) are supported.
               This will activate IP aliases. See the [official documentation](https://cloud.google.com/kubernetes-engine/docs/how-to/ip-aliases)
               Structure is documented below. This field is marked to use [Attribute as Block](https://www.terraform.io/docs/configuration/attr-as-blocks.html)
               in order to support explicit removal with `ip_allocation_policy = []`.
        :param pulumi.Input[str] location: The location (region or zone) in which the cluster
               master will be created, as well as the default node location. If you specify a
               zone (such as `us-central1-a`), the cluster will be a zonal cluster with a
               single cluster master. If you specify a region (such as `us-west1`), the
               cluster will be a regional cluster with multiple masters spread across zones in
               the region, and with default node locations in those zones as well.
        :param pulumi.Input[str] logging_service: The logging service that the cluster should
               write logs to. Available options include `logging.googleapis.com`,
               `logging.googleapis.com/kubernetes`, and `none`. Defaults to `logging.googleapis.com`
        :param pulumi.Input[dict] maintenance_policy: The maintenance policy to use for the cluster. Structure is
               documented below.
        :param pulumi.Input[dict] master_auth: The authentication information for accessing the
               Kubernetes master. Some values in this block are only returned by the API if
               your service account has permission to get credentials for your GKE cluster. If
               you see an unexpected diff removing a username/password or unsetting your client
               cert, ensure you have the `container.clusters.getCredentials` permission.
               Structure is documented below.
        :param pulumi.Input[dict] master_authorized_networks_config: The desired configuration options
               for master authorized networks. Omit the nested `cidr_blocks` attribute to disallow
               external access (except the cluster node IPs, which GKE automatically whitelists).
        :param pulumi.Input[str] min_master_version: The minimum version of the master. GKE
               will auto-update the master to new versions, so this does not guarantee the
               current master version--use the read-only `master_version` field to obtain that.
               If unset, the cluster's version will be set by GKE to the version of the most recent
               official release (which is not necessarily the latest version).  Most users will find
               the `container.getEngineVersions` data source useful - it indicates which versions
               are available, and can be use to approximate fuzzy versions. If you intend to specify versions manually,
               [the docs](https://cloud.google.com/kubernetes-engine/versioning-and-upgrades#specifying_cluster_version)
               describe the various acceptable formats for this field.
        :param pulumi.Input[str] monitoring_service: The monitoring service that the cluster
               should write metrics to.
               Automatically send metrics from pods in the cluster to the Google Cloud Monitoring API.
               VM metrics will be collected by Google Compute Engine regardless of this setting
               Available options include
               `monitoring.googleapis.com`, `monitoring.googleapis.com/kubernetes`, and `none`.
               Defaults to `monitoring.googleapis.com`
        :param pulumi.Input[str] name: The name of the cluster, unique within the project and
               location.
        :param pulumi.Input[str] network: The name or self_link of the Google Compute Engine
               network to which the cluster is connected. For Shared VPC, set this to the self link of the
               shared network.
        :param pulumi.Input[dict] network_policy: Configuration options for the
               [NetworkPolicy](https://kubernetes.io/docs/concepts/services-networking/networkpolicies/)
               feature. Structure is documented below.
        :param pulumi.Input[dict] node_config: Parameters used in creating the default node pool.
               Generally, this field should not be used at the same time as a
               `container.NodePool` or a `node_pool` block; this configuration
               manages the default node pool, which isn't recommended to be used with
               this provider. Structure is documented below.
        :param pulumi.Input[list] node_locations: The list of zones in which the cluster's nodes
               are located. Nodes must be in the region of their regional cluster or in the
               same region as their cluster's zone for zonal clusters. If this is specified for
               a zonal cluster, omit the cluster's zone.
        :param pulumi.Input[list] node_pools: List of node pools associated with this cluster.
               See container.NodePool for schema.
               **Warning:** node pools defined inside a cluster can't be changed (or added/removed) after
               cluster creation without deleting and recreating the entire cluster. Unless you absolutely need the ability
               to say "these are the _only_ node pools associated with this cluster", use the
               container.NodePool resource instead of this property.
        :param pulumi.Input[str] node_version: The Kubernetes version on the nodes. Must either be unset
               or set to the same value as `min_master_version` on create. Defaults to the default
               version set by GKE which is not necessarily the latest version. This only affects
               nodes in the default node pool. While a fuzzy version can be specified, it's
               recommended that you specify explicit versions as this provider will see spurious diffs
               when fuzzy versions are used. See the `container.getEngineVersions` data source's
               `version_prefix` field to approximate fuzzy versions.
               To update nodes in other node pools, use the `version` attribute on the node pool.
        :param pulumi.Input[dict] pod_security_policy_config: ) Configuration for the
               [PodSecurityPolicy](https://cloud.google.com/kubernetes-engine/docs/how-to/pod-security-policies) feature.
               Structure is documented below.
        :param pulumi.Input[dict] private_cluster_config: A set of options for creating
               a private cluster. Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[dict] release_channel: ) Configuration options for the
               [Release channel](https://cloud.google.com/kubernetes-engine/docs/concepts/release-channels)
               feature, which provide more control over automatic upgrades of your GKE clusters. Structure is documented below.
        :param pulumi.Input[bool] remove_default_node_pool: If `true`, deletes the default node
               pool upon cluster creation. If you're using `container.NodePool`
               resources with no default node pool, this should be set to `true`, alongside
               setting `initial_node_count` to at least `1`.
        :param pulumi.Input[dict] resource_labels: The GCE resource labels (a map of key/value pairs) to be applied to the cluster.
        :param pulumi.Input[dict] resource_usage_export_config: ) Configuration for the
               [ResourceUsageExportConfig](https://cloud.google.com/kubernetes-engine/docs/how-to/cluster-usage-metering) feature.
               Structure is documented below.
        :param pulumi.Input[str] subnetwork: The name or self_link of the Google Compute Engine subnetwork in
               which the cluster's instances are launched.
        :param pulumi.Input[dict] vertical_pod_autoscaling: )
               Vertical Pod Autoscaling automatically adjusts the resources of pods controlled by it.
               Structure is documented below.
        :param pulumi.Input[dict] workload_identity_config: )
               Workload Identity allows Kubernetes service accounts to act as a user-managed
               [Google IAM Service Account](https://cloud.google.com/iam/docs/service-accounts#user-managed_service_accounts).
               Structure is documented below.
        :param pulumi.Input[str] zone: The zone that the cluster master and nodes
               should be created in. If specified, this cluster will be a zonal cluster. `zone`
               has been deprecated in favour of `location`.
        
        The **addons_config** object supports the following:
        
          * `cloudrunConfig` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
          * `horizontalPodAutoscaling` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
          * `httpLoadBalancing` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
          * `istioConfig` (`pulumi.Input[dict]`)
        
            * `auth` (`pulumi.Input[str]`)
            * `disabled` (`pulumi.Input[bool]`)
        
          * `kubernetesDashboard` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
          * `networkPolicyConfig` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
        The **authenticator_groups_config** object supports the following:
        
          * `securityGroup` (`pulumi.Input[str]`)
        
        The **cluster_autoscaling** object supports the following:
        
          * `enabled` (`pulumi.Input[bool]`)
          * `resourceLimits` (`pulumi.Input[list]`)
        
            * `maximum` (`pulumi.Input[float]`)
            * `minimum` (`pulumi.Input[float]`)
            * `resourceType` (`pulumi.Input[str]`)
        
        The **database_encryption** object supports the following:
        
          * `keyName` (`pulumi.Input[str]`)
          * `state` (`pulumi.Input[str]`)
        
        The **ip_allocation_policy** object supports the following:
        
          * `clusterIpv4CidrBlock` (`pulumi.Input[str]`)
          * `clusterSecondaryRangeName` (`pulumi.Input[str]`)
          * `createSubnetwork` (`pulumi.Input[bool]`)
          * `nodeIpv4CidrBlock` (`pulumi.Input[str]`)
          * `servicesIpv4CidrBlock` (`pulumi.Input[str]`)
          * `servicesSecondaryRangeName` (`pulumi.Input[str]`)
          * `subnetworkName` (`pulumi.Input[str]`)
          * `useIpAliases` (`pulumi.Input[bool]`)
        
        The **maintenance_policy** object supports the following:
        
          * `dailyMaintenanceWindow` (`pulumi.Input[dict]`)
        
            * `duration` (`pulumi.Input[str]`)
            * `startTime` (`pulumi.Input[str]`)
        
        The **master_auth** object supports the following:
        
          * `clientCertificate` (`pulumi.Input[str]`)
          * `clientCertificateConfig` (`pulumi.Input[dict]`)
        
            * `issueClientCertificate` (`pulumi.Input[bool]`)
        
          * `clientKey` (`pulumi.Input[str]`)
          * `clusterCaCertificate` (`pulumi.Input[str]`)
          * `password` (`pulumi.Input[str]`)
          * `username` (`pulumi.Input[str]`)
        
        The **master_authorized_networks_config** object supports the following:
        
          * `cidrBlocks` (`pulumi.Input[list]`)
        
            * `cidrBlock` (`pulumi.Input[str]`)
            * `display_name` (`pulumi.Input[str]`)
        
        The **network_policy** object supports the following:
        
          * `enabled` (`pulumi.Input[bool]`)
          * `provider` (`pulumi.Input[str]`)
        
        The **node_config** object supports the following:
        
          * `disk_size_gb` (`pulumi.Input[float]`)
          * `diskType` (`pulumi.Input[str]`)
          * `guest_accelerators` (`pulumi.Input[list]`)
        
            * `count` (`pulumi.Input[float]`)
            * `type` (`pulumi.Input[str]`)
        
          * `imageType` (`pulumi.Input[str]`)
          * `labels` (`pulumi.Input[dict]`)
          * `localSsdCount` (`pulumi.Input[float]`)
          * `machine_type` (`pulumi.Input[str]`)
          * `metadata` (`pulumi.Input[dict]`)
          * `min_cpu_platform` (`pulumi.Input[str]`)
          * `oauthScopes` (`pulumi.Input[list]`)
          * `preemptible` (`pulumi.Input[bool]`)
          * `sandboxConfig` (`pulumi.Input[dict]`)
        
            * `sandboxType` (`pulumi.Input[str]`)
        
          * `service_account` (`pulumi.Input[str]`)
          * `shielded_instance_config` (`pulumi.Input[dict]`)
        
            * `enableIntegrityMonitoring` (`pulumi.Input[bool]`)
            * `enableSecureBoot` (`pulumi.Input[bool]`)
        
          * `tags` (`pulumi.Input[list]`)
          * `taints` (`pulumi.Input[list]`)
        
            * `effect` (`pulumi.Input[str]`)
            * `key` (`pulumi.Input[str]`)
            * `value` (`pulumi.Input[str]`)
        
          * `workloadMetadataConfig` (`pulumi.Input[dict]`)
        
            * `nodeMetadata` (`pulumi.Input[str]`)
        
        The **node_pools** object supports the following:
        
          * `autoscaling` (`pulumi.Input[dict]`)
        
            * `maxNodeCount` (`pulumi.Input[float]`)
            * `minNodeCount` (`pulumi.Input[float]`)
        
          * `initial_node_count` (`pulumi.Input[float]`) - The number of nodes to create in this
            cluster's default node pool. In regional or multi-zonal clusters, this is the
            number of nodes per zone. Must be set if `node_pool` is not set. If you're using
            `container.NodePool` objects with no default node pool, you'll need to
            set this to a value of at least `1`, alongside setting
            `remove_default_node_pool` to `true`.
          * `instance_group_urls` (`pulumi.Input[list]`) - List of instance group URLs which have been assigned
            to the cluster.
          * `management` (`pulumi.Input[dict]`)
        
            * `autoRepair` (`pulumi.Input[bool]`)
            * `autoUpgrade` (`pulumi.Input[bool]`)
        
          * `max_pods_per_node` (`pulumi.Input[float]`)
          * `name` (`pulumi.Input[str]`) - The name of the cluster, unique within the project and
            location.
          * `name_prefix` (`pulumi.Input[str]`)
          * `node_config` (`pulumi.Input[dict]`) - Parameters used in creating the default node pool.
            Generally, this field should not be used at the same time as a
            `container.NodePool` or a `node_pool` block; this configuration
            manages the default node pool, which isn't recommended to be used with
            this provider. Structure is documented below.
        
            * `disk_size_gb` (`pulumi.Input[float]`)
            * `diskType` (`pulumi.Input[str]`)
            * `guest_accelerators` (`pulumi.Input[list]`)
        
              * `count` (`pulumi.Input[float]`)
              * `type` (`pulumi.Input[str]`)
        
            * `imageType` (`pulumi.Input[str]`)
            * `labels` (`pulumi.Input[dict]`)
            * `localSsdCount` (`pulumi.Input[float]`)
            * `machine_type` (`pulumi.Input[str]`)
            * `metadata` (`pulumi.Input[dict]`)
            * `min_cpu_platform` (`pulumi.Input[str]`)
            * `oauthScopes` (`pulumi.Input[list]`)
            * `preemptible` (`pulumi.Input[bool]`)
            * `sandboxConfig` (`pulumi.Input[dict]`)
        
              * `sandboxType` (`pulumi.Input[str]`)
        
            * `service_account` (`pulumi.Input[str]`)
            * `shielded_instance_config` (`pulumi.Input[dict]`)
        
              * `enableIntegrityMonitoring` (`pulumi.Input[bool]`)
              * `enableSecureBoot` (`pulumi.Input[bool]`)
        
            * `tags` (`pulumi.Input[list]`)
            * `taints` (`pulumi.Input[list]`)
        
              * `effect` (`pulumi.Input[str]`)
              * `key` (`pulumi.Input[str]`)
              * `value` (`pulumi.Input[str]`)
        
            * `workloadMetadataConfig` (`pulumi.Input[dict]`)
        
              * `nodeMetadata` (`pulumi.Input[str]`)
        
          * `node_count` (`pulumi.Input[float]`)
          * `node_locations` (`pulumi.Input[list]`) - The list of zones in which the cluster's nodes
            are located. Nodes must be in the region of their regional cluster or in the
            same region as their cluster's zone for zonal clusters. If this is specified for
            a zonal cluster, omit the cluster's zone.
          * `version` (`pulumi.Input[str]`)
        
        The **pod_security_policy_config** object supports the following:
        
          * `enabled` (`pulumi.Input[bool]`)
        
        The **private_cluster_config** object supports the following:
        
          * `enablePrivateEndpoint` (`pulumi.Input[bool]`)
          * `enablePrivateNodes` (`pulumi.Input[bool]`)
          * `masterIpv4CidrBlock` (`pulumi.Input[str]`)
          * `privateEndpoint` (`pulumi.Input[str]`)
          * `publicEndpoint` (`pulumi.Input[str]`)
        
        The **release_channel** object supports the following:
        
          * `channel` (`pulumi.Input[str]`)
        
        The **resource_usage_export_config** object supports the following:
        
          * `bigqueryDestination` (`pulumi.Input[dict]`)
        
            * `dataset_id` (`pulumi.Input[str]`)
        
          * `enableNetworkEgressMetering` (`pulumi.Input[bool]`)
        
        The **vertical_pod_autoscaling** object supports the following:
        
          * `enabled` (`pulumi.Input[bool]`)
        
        The **workload_identity_config** object supports the following:
        
          * `identityNamespace` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/container_cluster.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['additional_zones'] = additional_zones
            __props__['addons_config'] = addons_config
            __props__['authenticator_groups_config'] = authenticator_groups_config
            __props__['cluster_autoscaling'] = cluster_autoscaling
            __props__['cluster_ipv4_cidr'] = cluster_ipv4_cidr
            __props__['database_encryption'] = database_encryption
            __props__['default_max_pods_per_node'] = default_max_pods_per_node
            __props__['description'] = description
            __props__['enable_binary_authorization'] = enable_binary_authorization
            __props__['enable_intranode_visibility'] = enable_intranode_visibility
            __props__['enable_kubernetes_alpha'] = enable_kubernetes_alpha
            __props__['enable_legacy_abac'] = enable_legacy_abac
            __props__['enable_shielded_nodes'] = enable_shielded_nodes
            __props__['enable_tpu'] = enable_tpu
            __props__['initial_node_count'] = initial_node_count
            __props__['ip_allocation_policy'] = ip_allocation_policy
            __props__['location'] = location
            __props__['logging_service'] = logging_service
            __props__['maintenance_policy'] = maintenance_policy
            __props__['master_auth'] = master_auth
            __props__['master_authorized_networks_config'] = master_authorized_networks_config
            __props__['min_master_version'] = min_master_version
            __props__['monitoring_service'] = monitoring_service
            __props__['name'] = name
            __props__['network'] = network
            __props__['network_policy'] = network_policy
            __props__['node_config'] = node_config
            __props__['node_locations'] = node_locations
            __props__['node_pools'] = node_pools
            __props__['node_version'] = node_version
            __props__['pod_security_policy_config'] = pod_security_policy_config
            __props__['private_cluster_config'] = private_cluster_config
            __props__['project'] = project
            __props__['region'] = region
            __props__['release_channel'] = release_channel
            __props__['remove_default_node_pool'] = remove_default_node_pool
            __props__['resource_labels'] = resource_labels
            __props__['resource_usage_export_config'] = resource_usage_export_config
            __props__['subnetwork'] = subnetwork
            __props__['vertical_pod_autoscaling'] = vertical_pod_autoscaling
            __props__['workload_identity_config'] = workload_identity_config
            __props__['zone'] = zone
            __props__['endpoint'] = None
            __props__['instance_group_urls'] = None
            __props__['master_version'] = None
            __props__['services_ipv4_cidr'] = None
            __props__['tpu_ipv4_cidr_block'] = None
        super(Cluster, __self__).__init__(
            'gcp:container/cluster:Cluster',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, additional_zones=None, addons_config=None, authenticator_groups_config=None, cluster_autoscaling=None, cluster_ipv4_cidr=None, database_encryption=None, default_max_pods_per_node=None, description=None, enable_binary_authorization=None, enable_intranode_visibility=None, enable_kubernetes_alpha=None, enable_legacy_abac=None, enable_shielded_nodes=None, enable_tpu=None, endpoint=None, initial_node_count=None, instance_group_urls=None, ip_allocation_policy=None, location=None, logging_service=None, maintenance_policy=None, master_auth=None, master_authorized_networks_config=None, master_version=None, min_master_version=None, monitoring_service=None, name=None, network=None, network_policy=None, node_config=None, node_locations=None, node_pools=None, node_version=None, pod_security_policy_config=None, private_cluster_config=None, project=None, region=None, release_channel=None, remove_default_node_pool=None, resource_labels=None, resource_usage_export_config=None, services_ipv4_cidr=None, subnetwork=None, tpu_ipv4_cidr_block=None, vertical_pod_autoscaling=None, workload_identity_config=None, zone=None):
        """
        Get an existing Cluster resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] additional_zones: The list of zones in which the cluster's nodes
               should be located. These must be in the same region as the cluster zone for
               zonal clusters, or in the region of a regional cluster. In a multi-zonal cluster,
               the number of nodes specified in `initial_node_count` is created in
               all specified zones as well as the primary zone. If specified for a regional
               cluster, nodes will only be created in these zones. `additional_zones` has been
               deprecated in favour of `node_locations`.
        :param pulumi.Input[dict] addons_config: The configuration for addons supported by GKE.
               Structure is documented below.
        :param pulumi.Input[dict] authenticator_groups_config: ) Configuration for the
               [Google Groups for GKE](https://cloud.google.com/kubernetes-engine/docs/how-to/role-based-access-control#groups-setup-gsuite) feature.
               Structure is documented below.
        :param pulumi.Input[dict] cluster_autoscaling: )
               Per-cluster configuration of Node Auto-Provisioning with Cluster Autoscaler to
               automatically adjust the size of the cluster and create/delete node pools based
               on the current needs of the cluster's workload. See the
               [guide to using Node Auto-Provisioning](https://cloud.google.com/kubernetes-engine/docs/how-to/node-auto-provisioning)
               for more details. Structure is documented below.
        :param pulumi.Input[str] cluster_ipv4_cidr: The IP address range of the Kubernetes pods
               in this cluster in CIDR notation (e.g. 10.96.0.0/14). Leave blank to have one
               automatically chosen or specify a /14 block in 10.0.0.0/8. This field will only
               work if your cluster is not VPC-native- when an `ip_allocation_policy` block is
               not defined, or `ip_allocation_policy.use_ip_aliases` is set to false. If your
               cluster is VPC-native, use `ip_allocation_policy.cluster_ipv4_cidr_block`.
        :param pulumi.Input[dict] database_encryption: ).
               Structure is documented below.
        :param pulumi.Input[float] default_max_pods_per_node: The default maximum number of pods
               per node in this cluster. This doesn't work on "routes-based" clusters, clusters
               that don't have IP Aliasing enabled. See the [official documentation](https://cloud.google.com/kubernetes-engine/docs/how-to/flexible-pod-cidr)
               for more information.
        :param pulumi.Input[str] description: Description of the cluster.
        :param pulumi.Input[bool] enable_binary_authorization: ) Enable Binary Authorization for this cluster.
               If enabled, all container images will be validated by Google Binary Authorization.
        :param pulumi.Input[bool] enable_intranode_visibility: )
               Whether Intra-node visibility is enabled for this cluster. This makes same node pod to pod traffic visible for VPC network.
        :param pulumi.Input[bool] enable_kubernetes_alpha: Whether to enable Kubernetes Alpha features for
               this cluster. Note that when this option is enabled, the cluster cannot be upgraded
               and will be automatically deleted after 30 days.
        :param pulumi.Input[bool] enable_legacy_abac: Whether the ABAC authorizer is enabled for this cluster.
               When enabled, identities in the system, including service accounts, nodes, and controllers,
               will have statically granted permissions beyond those provided by the RBAC configuration or IAM.
               Defaults to `false`
        :param pulumi.Input[bool] enable_shielded_nodes: ) Enable Shielded Nodes features on all nodes in this cluster.  Defaults to `false`.
        :param pulumi.Input[bool] enable_tpu: ) Whether to enable Cloud TPU resources in this cluster.
               See the [official documentation](https://cloud.google.com/tpu/docs/kubernetes-engine-setup).
        :param pulumi.Input[str] endpoint: The IP address of this cluster's Kubernetes master.
        :param pulumi.Input[float] initial_node_count: The number of nodes to create in this
               cluster's default node pool. In regional or multi-zonal clusters, this is the
               number of nodes per zone. Must be set if `node_pool` is not set. If you're using
               `container.NodePool` objects with no default node pool, you'll need to
               set this to a value of at least `1`, alongside setting
               `remove_default_node_pool` to `true`.
        :param pulumi.Input[list] instance_group_urls: List of instance group URLs which have been assigned
               to the cluster.
        :param pulumi.Input[dict] ip_allocation_policy: Configuration for cluster IP allocation. As of now, only pre-allocated subnetworks (custom type with secondary ranges) are supported.
               This will activate IP aliases. See the [official documentation](https://cloud.google.com/kubernetes-engine/docs/how-to/ip-aliases)
               Structure is documented below. This field is marked to use [Attribute as Block](https://www.terraform.io/docs/configuration/attr-as-blocks.html)
               in order to support explicit removal with `ip_allocation_policy = []`.
        :param pulumi.Input[str] location: The location (region or zone) in which the cluster
               master will be created, as well as the default node location. If you specify a
               zone (such as `us-central1-a`), the cluster will be a zonal cluster with a
               single cluster master. If you specify a region (such as `us-west1`), the
               cluster will be a regional cluster with multiple masters spread across zones in
               the region, and with default node locations in those zones as well.
        :param pulumi.Input[str] logging_service: The logging service that the cluster should
               write logs to. Available options include `logging.googleapis.com`,
               `logging.googleapis.com/kubernetes`, and `none`. Defaults to `logging.googleapis.com`
        :param pulumi.Input[dict] maintenance_policy: The maintenance policy to use for the cluster. Structure is
               documented below.
        :param pulumi.Input[dict] master_auth: The authentication information for accessing the
               Kubernetes master. Some values in this block are only returned by the API if
               your service account has permission to get credentials for your GKE cluster. If
               you see an unexpected diff removing a username/password or unsetting your client
               cert, ensure you have the `container.clusters.getCredentials` permission.
               Structure is documented below.
        :param pulumi.Input[dict] master_authorized_networks_config: The desired configuration options
               for master authorized networks. Omit the nested `cidr_blocks` attribute to disallow
               external access (except the cluster node IPs, which GKE automatically whitelists).
        :param pulumi.Input[str] master_version: The current version of the master in the cluster. This may
               be different than the `min_master_version` set in the config if the master
               has been updated by GKE.
        :param pulumi.Input[str] min_master_version: The minimum version of the master. GKE
               will auto-update the master to new versions, so this does not guarantee the
               current master version--use the read-only `master_version` field to obtain that.
               If unset, the cluster's version will be set by GKE to the version of the most recent
               official release (which is not necessarily the latest version).  Most users will find
               the `container.getEngineVersions` data source useful - it indicates which versions
               are available, and can be use to approximate fuzzy versions. If you intend to specify versions manually,
               [the docs](https://cloud.google.com/kubernetes-engine/versioning-and-upgrades#specifying_cluster_version)
               describe the various acceptable formats for this field.
        :param pulumi.Input[str] monitoring_service: The monitoring service that the cluster
               should write metrics to.
               Automatically send metrics from pods in the cluster to the Google Cloud Monitoring API.
               VM metrics will be collected by Google Compute Engine regardless of this setting
               Available options include
               `monitoring.googleapis.com`, `monitoring.googleapis.com/kubernetes`, and `none`.
               Defaults to `monitoring.googleapis.com`
        :param pulumi.Input[str] name: The name of the cluster, unique within the project and
               location.
        :param pulumi.Input[str] network: The name or self_link of the Google Compute Engine
               network to which the cluster is connected. For Shared VPC, set this to the self link of the
               shared network.
        :param pulumi.Input[dict] network_policy: Configuration options for the
               [NetworkPolicy](https://kubernetes.io/docs/concepts/services-networking/networkpolicies/)
               feature. Structure is documented below.
        :param pulumi.Input[dict] node_config: Parameters used in creating the default node pool.
               Generally, this field should not be used at the same time as a
               `container.NodePool` or a `node_pool` block; this configuration
               manages the default node pool, which isn't recommended to be used with
               this provider. Structure is documented below.
        :param pulumi.Input[list] node_locations: The list of zones in which the cluster's nodes
               are located. Nodes must be in the region of their regional cluster or in the
               same region as their cluster's zone for zonal clusters. If this is specified for
               a zonal cluster, omit the cluster's zone.
        :param pulumi.Input[list] node_pools: List of node pools associated with this cluster.
               See container.NodePool for schema.
               **Warning:** node pools defined inside a cluster can't be changed (or added/removed) after
               cluster creation without deleting and recreating the entire cluster. Unless you absolutely need the ability
               to say "these are the _only_ node pools associated with this cluster", use the
               container.NodePool resource instead of this property.
        :param pulumi.Input[str] node_version: The Kubernetes version on the nodes. Must either be unset
               or set to the same value as `min_master_version` on create. Defaults to the default
               version set by GKE which is not necessarily the latest version. This only affects
               nodes in the default node pool. While a fuzzy version can be specified, it's
               recommended that you specify explicit versions as this provider will see spurious diffs
               when fuzzy versions are used. See the `container.getEngineVersions` data source's
               `version_prefix` field to approximate fuzzy versions.
               To update nodes in other node pools, use the `version` attribute on the node pool.
        :param pulumi.Input[dict] pod_security_policy_config: ) Configuration for the
               [PodSecurityPolicy](https://cloud.google.com/kubernetes-engine/docs/how-to/pod-security-policies) feature.
               Structure is documented below.
        :param pulumi.Input[dict] private_cluster_config: A set of options for creating
               a private cluster. Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[dict] release_channel: ) Configuration options for the
               [Release channel](https://cloud.google.com/kubernetes-engine/docs/concepts/release-channels)
               feature, which provide more control over automatic upgrades of your GKE clusters. Structure is documented below.
        :param pulumi.Input[bool] remove_default_node_pool: If `true`, deletes the default node
               pool upon cluster creation. If you're using `container.NodePool`
               resources with no default node pool, this should be set to `true`, alongside
               setting `initial_node_count` to at least `1`.
        :param pulumi.Input[dict] resource_labels: The GCE resource labels (a map of key/value pairs) to be applied to the cluster.
        :param pulumi.Input[dict] resource_usage_export_config: ) Configuration for the
               [ResourceUsageExportConfig](https://cloud.google.com/kubernetes-engine/docs/how-to/cluster-usage-metering) feature.
               Structure is documented below.
        :param pulumi.Input[str] services_ipv4_cidr: The IP address range of the Kubernetes services in this
               cluster, in [CIDR](http://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing)
               notation (e.g. `1.2.3.4/29`). Service addresses are typically put in the last
               `/16` from the container CIDR.
        :param pulumi.Input[str] subnetwork: The name or self_link of the Google Compute Engine subnetwork in
               which the cluster's instances are launched.
        :param pulumi.Input[dict] vertical_pod_autoscaling: )
               Vertical Pod Autoscaling automatically adjusts the resources of pods controlled by it.
               Structure is documented below.
        :param pulumi.Input[dict] workload_identity_config: )
               Workload Identity allows Kubernetes service accounts to act as a user-managed
               [Google IAM Service Account](https://cloud.google.com/iam/docs/service-accounts#user-managed_service_accounts).
               Structure is documented below.
        :param pulumi.Input[str] zone: The zone that the cluster master and nodes
               should be created in. If specified, this cluster will be a zonal cluster. `zone`
               has been deprecated in favour of `location`.
        
        The **addons_config** object supports the following:
        
          * `cloudrunConfig` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
          * `horizontalPodAutoscaling` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
          * `httpLoadBalancing` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
          * `istioConfig` (`pulumi.Input[dict]`)
        
            * `auth` (`pulumi.Input[str]`)
            * `disabled` (`pulumi.Input[bool]`)
        
          * `kubernetesDashboard` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
          * `networkPolicyConfig` (`pulumi.Input[dict]`)
        
            * `disabled` (`pulumi.Input[bool]`)
        
        The **authenticator_groups_config** object supports the following:
        
          * `securityGroup` (`pulumi.Input[str]`)
        
        The **cluster_autoscaling** object supports the following:
        
          * `enabled` (`pulumi.Input[bool]`)
          * `resourceLimits` (`pulumi.Input[list]`)
        
            * `maximum` (`pulumi.Input[float]`)
            * `minimum` (`pulumi.Input[float]`)
            * `resourceType` (`pulumi.Input[str]`)
        
        The **database_encryption** object supports the following:
        
          * `keyName` (`pulumi.Input[str]`)
          * `state` (`pulumi.Input[str]`)
        
        The **ip_allocation_policy** object supports the following:
        
          * `clusterIpv4CidrBlock` (`pulumi.Input[str]`)
          * `clusterSecondaryRangeName` (`pulumi.Input[str]`)
          * `createSubnetwork` (`pulumi.Input[bool]`)
          * `nodeIpv4CidrBlock` (`pulumi.Input[str]`)
          * `servicesIpv4CidrBlock` (`pulumi.Input[str]`)
          * `servicesSecondaryRangeName` (`pulumi.Input[str]`)
          * `subnetworkName` (`pulumi.Input[str]`)
          * `useIpAliases` (`pulumi.Input[bool]`)
        
        The **maintenance_policy** object supports the following:
        
          * `dailyMaintenanceWindow` (`pulumi.Input[dict]`)
        
            * `duration` (`pulumi.Input[str]`)
            * `startTime` (`pulumi.Input[str]`)
        
        The **master_auth** object supports the following:
        
          * `clientCertificate` (`pulumi.Input[str]`)
          * `clientCertificateConfig` (`pulumi.Input[dict]`)
        
            * `issueClientCertificate` (`pulumi.Input[bool]`)
        
          * `clientKey` (`pulumi.Input[str]`)
          * `clusterCaCertificate` (`pulumi.Input[str]`)
          * `password` (`pulumi.Input[str]`)
          * `username` (`pulumi.Input[str]`)
        
        The **master_authorized_networks_config** object supports the following:
        
          * `cidrBlocks` (`pulumi.Input[list]`)
        
            * `cidrBlock` (`pulumi.Input[str]`)
            * `display_name` (`pulumi.Input[str]`)
        
        The **network_policy** object supports the following:
        
          * `enabled` (`pulumi.Input[bool]`)
          * `provider` (`pulumi.Input[str]`)
        
        The **node_config** object supports the following:
        
          * `disk_size_gb` (`pulumi.Input[float]`)
          * `diskType` (`pulumi.Input[str]`)
          * `guest_accelerators` (`pulumi.Input[list]`)
        
            * `count` (`pulumi.Input[float]`)
            * `type` (`pulumi.Input[str]`)
        
          * `imageType` (`pulumi.Input[str]`)
          * `labels` (`pulumi.Input[dict]`)
          * `localSsdCount` (`pulumi.Input[float]`)
          * `machine_type` (`pulumi.Input[str]`)
          * `metadata` (`pulumi.Input[dict]`)
          * `min_cpu_platform` (`pulumi.Input[str]`)
          * `oauthScopes` (`pulumi.Input[list]`)
          * `preemptible` (`pulumi.Input[bool]`)
          * `sandboxConfig` (`pulumi.Input[dict]`)
        
            * `sandboxType` (`pulumi.Input[str]`)
        
          * `service_account` (`pulumi.Input[str]`)
          * `shielded_instance_config` (`pulumi.Input[dict]`)
        
            * `enableIntegrityMonitoring` (`pulumi.Input[bool]`)
            * `enableSecureBoot` (`pulumi.Input[bool]`)
        
          * `tags` (`pulumi.Input[list]`)
          * `taints` (`pulumi.Input[list]`)
        
            * `effect` (`pulumi.Input[str]`)
            * `key` (`pulumi.Input[str]`)
            * `value` (`pulumi.Input[str]`)
        
          * `workloadMetadataConfig` (`pulumi.Input[dict]`)
        
            * `nodeMetadata` (`pulumi.Input[str]`)
        
        The **node_pools** object supports the following:
        
          * `autoscaling` (`pulumi.Input[dict]`)
        
            * `maxNodeCount` (`pulumi.Input[float]`)
            * `minNodeCount` (`pulumi.Input[float]`)
        
          * `initial_node_count` (`pulumi.Input[float]`) - The number of nodes to create in this
            cluster's default node pool. In regional or multi-zonal clusters, this is the
            number of nodes per zone. Must be set if `node_pool` is not set. If you're using
            `container.NodePool` objects with no default node pool, you'll need to
            set this to a value of at least `1`, alongside setting
            `remove_default_node_pool` to `true`.
          * `instance_group_urls` (`pulumi.Input[list]`) - List of instance group URLs which have been assigned
            to the cluster.
          * `management` (`pulumi.Input[dict]`)
        
            * `autoRepair` (`pulumi.Input[bool]`)
            * `autoUpgrade` (`pulumi.Input[bool]`)
        
          * `max_pods_per_node` (`pulumi.Input[float]`)
          * `name` (`pulumi.Input[str]`) - The name of the cluster, unique within the project and
            location.
          * `name_prefix` (`pulumi.Input[str]`)
          * `node_config` (`pulumi.Input[dict]`) - Parameters used in creating the default node pool.
            Generally, this field should not be used at the same time as a
            `container.NodePool` or a `node_pool` block; this configuration
            manages the default node pool, which isn't recommended to be used with
            this provider. Structure is documented below.
        
            * `disk_size_gb` (`pulumi.Input[float]`)
            * `diskType` (`pulumi.Input[str]`)
            * `guest_accelerators` (`pulumi.Input[list]`)
        
              * `count` (`pulumi.Input[float]`)
              * `type` (`pulumi.Input[str]`)
        
            * `imageType` (`pulumi.Input[str]`)
            * `labels` (`pulumi.Input[dict]`)
            * `localSsdCount` (`pulumi.Input[float]`)
            * `machine_type` (`pulumi.Input[str]`)
            * `metadata` (`pulumi.Input[dict]`)
            * `min_cpu_platform` (`pulumi.Input[str]`)
            * `oauthScopes` (`pulumi.Input[list]`)
            * `preemptible` (`pulumi.Input[bool]`)
            * `sandboxConfig` (`pulumi.Input[dict]`)
        
              * `sandboxType` (`pulumi.Input[str]`)
        
            * `service_account` (`pulumi.Input[str]`)
            * `shielded_instance_config` (`pulumi.Input[dict]`)
        
              * `enableIntegrityMonitoring` (`pulumi.Input[bool]`)
              * `enableSecureBoot` (`pulumi.Input[bool]`)
        
            * `tags` (`pulumi.Input[list]`)
            * `taints` (`pulumi.Input[list]`)
        
              * `effect` (`pulumi.Input[str]`)
              * `key` (`pulumi.Input[str]`)
              * `value` (`pulumi.Input[str]`)
        
            * `workloadMetadataConfig` (`pulumi.Input[dict]`)
        
              * `nodeMetadata` (`pulumi.Input[str]`)
        
          * `node_count` (`pulumi.Input[float]`)
          * `node_locations` (`pulumi.Input[list]`) - The list of zones in which the cluster's nodes
            are located. Nodes must be in the region of their regional cluster or in the
            same region as their cluster's zone for zonal clusters. If this is specified for
            a zonal cluster, omit the cluster's zone.
          * `version` (`pulumi.Input[str]`)
        
        The **pod_security_policy_config** object supports the following:
        
          * `enabled` (`pulumi.Input[bool]`)
        
        The **private_cluster_config** object supports the following:
        
          * `enablePrivateEndpoint` (`pulumi.Input[bool]`)
          * `enablePrivateNodes` (`pulumi.Input[bool]`)
          * `masterIpv4CidrBlock` (`pulumi.Input[str]`)
          * `privateEndpoint` (`pulumi.Input[str]`)
          * `publicEndpoint` (`pulumi.Input[str]`)
        
        The **release_channel** object supports the following:
        
          * `channel` (`pulumi.Input[str]`)
        
        The **resource_usage_export_config** object supports the following:
        
          * `bigqueryDestination` (`pulumi.Input[dict]`)
        
            * `dataset_id` (`pulumi.Input[str]`)
        
          * `enableNetworkEgressMetering` (`pulumi.Input[bool]`)
        
        The **vertical_pod_autoscaling** object supports the following:
        
          * `enabled` (`pulumi.Input[bool]`)
        
        The **workload_identity_config** object supports the following:
        
          * `identityNamespace` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/container_cluster.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["additional_zones"] = additional_zones
        __props__["addons_config"] = addons_config
        __props__["authenticator_groups_config"] = authenticator_groups_config
        __props__["cluster_autoscaling"] = cluster_autoscaling
        __props__["cluster_ipv4_cidr"] = cluster_ipv4_cidr
        __props__["database_encryption"] = database_encryption
        __props__["default_max_pods_per_node"] = default_max_pods_per_node
        __props__["description"] = description
        __props__["enable_binary_authorization"] = enable_binary_authorization
        __props__["enable_intranode_visibility"] = enable_intranode_visibility
        __props__["enable_kubernetes_alpha"] = enable_kubernetes_alpha
        __props__["enable_legacy_abac"] = enable_legacy_abac
        __props__["enable_shielded_nodes"] = enable_shielded_nodes
        __props__["enable_tpu"] = enable_tpu
        __props__["endpoint"] = endpoint
        __props__["initial_node_count"] = initial_node_count
        __props__["instance_group_urls"] = instance_group_urls
        __props__["ip_allocation_policy"] = ip_allocation_policy
        __props__["location"] = location
        __props__["logging_service"] = logging_service
        __props__["maintenance_policy"] = maintenance_policy
        __props__["master_auth"] = master_auth
        __props__["master_authorized_networks_config"] = master_authorized_networks_config
        __props__["master_version"] = master_version
        __props__["min_master_version"] = min_master_version
        __props__["monitoring_service"] = monitoring_service
        __props__["name"] = name
        __props__["network"] = network
        __props__["network_policy"] = network_policy
        __props__["node_config"] = node_config
        __props__["node_locations"] = node_locations
        __props__["node_pools"] = node_pools
        __props__["node_version"] = node_version
        __props__["pod_security_policy_config"] = pod_security_policy_config
        __props__["private_cluster_config"] = private_cluster_config
        __props__["project"] = project
        __props__["region"] = region
        __props__["release_channel"] = release_channel
        __props__["remove_default_node_pool"] = remove_default_node_pool
        __props__["resource_labels"] = resource_labels
        __props__["resource_usage_export_config"] = resource_usage_export_config
        __props__["services_ipv4_cidr"] = services_ipv4_cidr
        __props__["subnetwork"] = subnetwork
        __props__["tpu_ipv4_cidr_block"] = tpu_ipv4_cidr_block
        __props__["vertical_pod_autoscaling"] = vertical_pod_autoscaling
        __props__["workload_identity_config"] = workload_identity_config
        __props__["zone"] = zone
        return Cluster(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class RegionBackendService(pulumi.CustomResource):
    backends: pulumi.Output[list]
    connection_draining_timeout_sec: pulumi.Output[float]
    description: pulumi.Output[str]
    failover_policy: pulumi.Output[dict]
    fingerprint: pulumi.Output[str]
    health_checks: pulumi.Output[str]
    load_balancing_scheme: pulumi.Output[str]
    name: pulumi.Output[str]
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    protocol: pulumi.Output[str]
    region: pulumi.Output[str]
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    session_affinity: pulumi.Output[str]
    timeout_sec: pulumi.Output[float]
    def __init__(__self__, resource_name, opts=None, backends=None, connection_draining_timeout_sec=None, description=None, failover_policy=None, health_checks=None, load_balancing_scheme=None, name=None, project=None, protocol=None, region=None, session_affinity=None, timeout_sec=None, __props__=None, __name__=None, __opts__=None):
        """
        Create a RegionBackendService resource with the given unique name, props, and options.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        
        The **backends** object supports the following:
        
          * `description` (`pulumi.Input[str]`)
          * `failover` (`pulumi.Input[bool]`)
          * `group` (`pulumi.Input[str]`)
        
        The **failover_policy** object supports the following:
        
          * `disableConnectionDrainOnFailover` (`pulumi.Input[bool]`)
          * `dropTrafficIfUnhealthy` (`pulumi.Input[bool]`)
          * `failover_ratio` (`pulumi.Input[float]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/compute_region_backend_service.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['backends'] = backends
            __props__['connection_draining_timeout_sec'] = connection_draining_timeout_sec
            __props__['description'] = description
            __props__['failover_policy'] = failover_policy
            if health_checks is None:
                raise TypeError("Missing required property 'health_checks'")
            __props__['health_checks'] = health_checks
            __props__['load_balancing_scheme'] = load_balancing_scheme
            __props__['name'] = name
            __props__['project'] = project
            __props__['protocol'] = protocol
            __props__['region'] = region
            __props__['session_affinity'] = session_affinity
            __props__['timeout_sec'] = timeout_sec
            __props__['fingerprint'] = None
            __props__['self_link'] = None
        super(RegionBackendService, __self__).__init__(
            'gcp:compute/regionBackendService:RegionBackendService',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, backends=None, connection_draining_timeout_sec=None, description=None, failover_policy=None, fingerprint=None, health_checks=None, load_balancing_scheme=None, name=None, project=None, protocol=None, region=None, self_link=None, session_affinity=None, timeout_sec=None):
        """
        Get an existing RegionBackendService resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] self_link: The URI of the created resource.
        
        The **backends** object supports the following:
        
          * `description` (`pulumi.Input[str]`)
          * `failover` (`pulumi.Input[bool]`)
          * `group` (`pulumi.Input[str]`)
        
        The **failover_policy** object supports the following:
        
          * `disableConnectionDrainOnFailover` (`pulumi.Input[bool]`)
          * `dropTrafficIfUnhealthy` (`pulumi.Input[bool]`)
          * `failover_ratio` (`pulumi.Input[float]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/compute_region_backend_service.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["backends"] = backends
        __props__["connection_draining_timeout_sec"] = connection_draining_timeout_sec
        __props__["description"] = description
        __props__["failover_policy"] = failover_policy
        __props__["fingerprint"] = fingerprint
        __props__["health_checks"] = health_checks
        __props__["load_balancing_scheme"] = load_balancing_scheme
        __props__["name"] = name
        __props__["project"] = project
        __props__["protocol"] = protocol
        __props__["region"] = region
        __props__["self_link"] = self_link
        __props__["session_affinity"] = session_affinity
        __props__["timeout_sec"] = timeout_sec
        return RegionBackendService(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class CryptoKey(pulumi.CustomResource):
    key_ring: pulumi.Output[str]
    labels: pulumi.Output[dict]
    name: pulumi.Output[str]
    purpose: pulumi.Output[str]
    rotation_period: pulumi.Output[str]
    self_link: pulumi.Output[str]
    version_template: pulumi.Output[dict]
    def __init__(__self__, resource_name, opts=None, key_ring=None, labels=None, name=None, purpose=None, rotation_period=None, version_template=None, __props__=None, __name__=None, __opts__=None):
        """
        A `CryptoKey` represents a logical key that can be used for cryptographic operations.
        
        
        > **Note:** CryptoKeys cannot be deleted from Google Cloud Platform.
        Destroying a CryptoKey will remove it from state
        and delete all CryptoKeyVersions, rendering the key unusable, but *will
        not delete the resource on the server.* When this provider destroys these keys,
        any data previously encrypted with these keys will be irrecoverable.
        For this reason, it is strongly recommended that you add lifecycle hooks
        to the resource to prevent accidental destruction.
        
        
        To get more information about CryptoKey, see:
        
        * [API documentation](https://cloud.google.com/kms/docs/reference/rest/v1/projects.locations.keyRings.cryptoKeys)
        * How-to Guides
            * [Creating a key](https://cloud.google.com/kms/docs/creating-keys#create_a_key)
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        
        The **version_template** object supports the following:
        
          * `algorithm` (`pulumi.Input[str]`)
          * `protectionLevel` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/kms_crypto_key.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if key_ring is None:
                raise TypeError("Missing required property 'key_ring'")
            __props__['key_ring'] = key_ring
            __props__['labels'] = labels
            __props__['name'] = name
            __props__['purpose'] = purpose
            __props__['rotation_period'] = rotation_period
            __props__['version_template'] = version_template
            __props__['self_link'] = None
        super(CryptoKey, __self__).__init__(
            'gcp:kms/cryptoKey:CryptoKey',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, key_ring=None, labels=None, name=None, purpose=None, rotation_period=None, self_link=None, version_template=None):
        """
        Get an existing CryptoKey resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        
        The **version_template** object supports the following:
        
          * `algorithm` (`pulumi.Input[str]`)
          * `protectionLevel` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/kms_crypto_key.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["key_ring"] = key_ring
        __props__["labels"] = labels
        __props__["name"] = name
        __props__["purpose"] = purpose
        __props__["rotation_period"] = rotation_period
        __props__["self_link"] = self_link
        __props__["version_template"] = version_template
        return CryptoKey(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetNetblockIPRangesResult:
    """
    A collection of values returned by getNetblockIPRanges.
    """
    def __init__(__self__, cidr_blocks=None, cidr_blocks_ipv4s=None, cidr_blocks_ipv6s=None, range_type=None, id=None):
        if cidr_blocks and not isinstance(cidr_blocks, list):
            raise TypeError("Expected argument 'cidr_blocks' to be a list")
        __self__.cidr_blocks = cidr_blocks
        """
        Retrieve list of all CIDR blocks.
        """
        if cidr_blocks_ipv4s and not isinstance(cidr_blocks_ipv4s, list):
            raise TypeError("Expected argument 'cidr_blocks_ipv4s' to be a list")
        __self__.cidr_blocks_ipv4s = cidr_blocks_ipv4s
        """
        Retrieve list of the IPv4 CIDR blocks
        """
        if cidr_blocks_ipv6s and not isinstance(cidr_blocks_ipv6s, list):
            raise TypeError("Expected argument 'cidr_blocks_ipv6s' to be a list")
        __self__.cidr_blocks_ipv6s = cidr_blocks_ipv6s
        """
        Retrieve list of the IPv6 CIDR blocks, if available.
        """
        if range_type and not isinstance(range_type, str):
            raise TypeError("Expected argument 'range_type' to be a str")
        __self__.range_type = range_type
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetNetblockIPRangesResult(GetNetblockIPRangesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetNetblockIPRangesResult(
            cidr_blocks=self.cidr_blocks,
            cidr_blocks_ipv4s=self.cidr_blocks_ipv4s,
            cidr_blocks_ipv6s=self.cidr_blocks_ipv6s,
            range_type=self.range_type,
            id=self.id)

def get_netblock_ip_ranges(range_type=None,opts=None):
    """
    Use this data source to get the IP addresses from different special IP ranges on Google Cloud Platform.
    
    :param str range_type: The type of range for which to provide results.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/d/netblock_ip_ranges.html.markdown.
    """
    __args__ = dict()

    __args__['rangeType'] = range_type
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('gcp:compute/getNetblockIPRanges:getNetblockIPRanges', __args__, opts=opts).value

    return AwaitableGetNetblockIPRangesResult(
        cidr_blocks=__ret__.get('cidrBlocks'),
        cidr_blocks_ipv4s=__ret__.get('cidrBlocksIpv4s'),
        cidr_blocks_ipv6s=__ret__.get('cidrBlocksIpv6s'),
        range_type=__ret__.get('rangeType'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetAccountAccessTokenResult:
    """
    A collection of values returned by getAccountAccessToken.
    """
    def __init__(__self__, access_token=None, delegates=None, lifetime=None, scopes=None, target_service_account=None, id=None):
        if access_token and not isinstance(access_token, str):
            raise TypeError("Expected argument 'access_token' to be a str")
        __self__.access_token = access_token
        """
        The `access_token` representing the new generated identity.
        """
        if delegates and not isinstance(delegates, list):
            raise TypeError("Expected argument 'delegates' to be a list")
        __self__.delegates = delegates
        if lifetime and not isinstance(lifetime, str):
            raise TypeError("Expected argument 'lifetime' to be a str")
        __self__.lifetime = lifetime
        if scopes and not isinstance(scopes, list):
            raise TypeError("Expected argument 'scopes' to be a list")
        __self__.scopes = scopes
        if target_service_account and not isinstance(target_service_account, str):
            raise TypeError("Expected argument 'target_service_account' to be a str")
        __self__.target_service_account = target_service_account
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetAccountAccessTokenResult(GetAccountAccessTokenResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAccountAccessTokenResult(
            access_token=self.access_token,
            delegates=self.delegates,
            lifetime=self.lifetime,
            scopes=self.scopes,
            target_service_account=self.target_service_account,
            id=self.id)

def get_account_access_token(delegates=None,lifetime=None,scopes=None,target_service_account=None,opts=None):
    """
    This data source provides a google `oauth2` `access_token` for a different service account than the one initially running the script.
    
    For more information see
    [the official documentation](https://cloud.google.com/iam/docs/creating-short-lived-service-account-credentials) as well as [iamcredentials.generateAccessToken()](https://cloud.google.com/iam/credentials/reference/rest/v1/projects.serviceAccounts/generateAccessToken)
    
    :param list delegates: Delegate chain of approvals needed to perform full impersonation. Specify the fully qualified service account name.  (e.g. `["projects/-/serviceAccounts/delegate-svc-account@project-id.iam.gserviceaccount.com"]`)
    :param str lifetime: Lifetime of the impersonated token (defaults to its max: `3600s`).
    :param list scopes: The scopes the new credential should have (e.g. `["storage-ro", "cloud-platform"]`)
    :param str target_service_account: The service account _to_ impersonate (e.g. `service_B@your-project-id.iam.gserviceaccount.com`)

    > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/d/service_account_access_token.html.markdown.
    """
    __args__ = dict()

    __args__['delegates'] = delegates
    __args__['lifetime'] = lifetime
    __args__['scopes'] = scopes
    __args__['targetServiceAccount'] = target_service_account
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('gcp:serviceAccount/getAccountAccessToken:getAccountAccessToken', __args__, opts=opts).value

    return AwaitableGetAccountAccessTokenResult(
        access_token=__ret__.get('accessToken'),
        delegates=__ret__.get('delegates'),
        lifetime=__ret__.get('lifetime'),
        scopes=__ret__.get('scopes'),
        target_service_account=__ret__.get('targetServiceAccount'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Instance(pulumi.CustomResource):
    clusters: pulumi.Output[list]
    """
    A block of cluster configuration options. This can be specified 1 or 2 times. See structure below.
    
      * `clusterId` (`str`)
      * `numNodes` (`float`)
      * `storageType` (`str`)
      * `zone` (`str`)
    """
    display_name: pulumi.Output[str]
    """
    The human-readable display name of the Bigtable instance. Defaults to the instance `name`.
    """
    instance_type: pulumi.Output[str]
    """
    The instance type to create. One of `"DEVELOPMENT"` or `"PRODUCTION"`. Defaults to `"PRODUCTION"`.
    """
    name: pulumi.Output[str]
    """
    The name (also called Instance Id in the Cloud Console) of the Cloud Bigtable instance.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs. If it
    is not provided, the provider project is used.
    """
    def __init__(__self__, resource_name, opts=None, clusters=None, display_name=None, instance_type=None, name=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        Creates a Google Bigtable instance. For more information see
        [the official documentation](https://cloud.google.com/bigtable/) and
        [API](https://cloud.google.com/bigtable/docs/go/reference).
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] clusters: A block of cluster configuration options. This can be specified 1 or 2 times. See structure below.
        :param pulumi.Input[str] display_name: The human-readable display name of the Bigtable instance. Defaults to the instance `name`.
        :param pulumi.Input[str] instance_type: The instance type to create. One of `"DEVELOPMENT"` or `"PRODUCTION"`. Defaults to `"PRODUCTION"`.
        :param pulumi.Input[str] name: The name (also called Instance Id in the Cloud Console) of the Cloud Bigtable instance.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        
        The **clusters** object supports the following:
        
          * `clusterId` (`pulumi.Input[str]`)
          * `numNodes` (`pulumi.Input[float]`)
          * `storageType` (`pulumi.Input[str]`)
          * `zone` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/bigtable_instance.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['clusters'] = clusters
            __props__['display_name'] = display_name
            __props__['instance_type'] = instance_type
            __props__['name'] = name
            __props__['project'] = project
        super(Instance, __self__).__init__(
            'gcp:bigtable/instance:Instance',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, clusters=None, display_name=None, instance_type=None, name=None, project=None):
        """
        Get an existing Instance resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] clusters: A block of cluster configuration options. This can be specified 1 or 2 times. See structure below.
        :param pulumi.Input[str] display_name: The human-readable display name of the Bigtable instance. Defaults to the instance `name`.
        :param pulumi.Input[str] instance_type: The instance type to create. One of `"DEVELOPMENT"` or `"PRODUCTION"`. Defaults to `"PRODUCTION"`.
        :param pulumi.Input[str] name: The name (also called Instance Id in the Cloud Console) of the Cloud Bigtable instance.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        
        The **clusters** object supports the following:
        
          * `clusterId` (`pulumi.Input[str]`)
          * `numNodes` (`pulumi.Input[float]`)
          * `storageType` (`pulumi.Input[str]`)
          * `zone` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/bigtable_instance.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["clusters"] = clusters
        __props__["display_name"] = display_name
        __props__["instance_type"] = instance_type
        __props__["name"] = name
        __props__["project"] = project
        return Instance(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Application(pulumi.CustomResource):
    app_id: pulumi.Output[str]
    """
    Identifier of the app, usually `{PROJECT_ID}`
    """
    auth_domain: pulumi.Output[str]
    """
    The domain to authenticate users with when using App Engine's User API.
    """
    code_bucket: pulumi.Output[str]
    """
    The GCS bucket code is being stored in for this app.
    """
    default_bucket: pulumi.Output[str]
    """
    The GCS bucket content is being stored in for this app.
    """
    default_hostname: pulumi.Output[str]
    """
    The default hostname for this app.
    """
    feature_settings: pulumi.Output[dict]
    """
    A block of optional settings to configure specific App Engine features:
    
      * `splitHealthChecks` (`bool`) - Set to false to use the legacy health check instead of the readiness
        and liveness checks.
    """
    gcr_domain: pulumi.Output[str]
    """
    The GCR domain used for storing managed Docker images for this app.
    """
    location_id: pulumi.Output[str]
    """
    The [location](https://cloud.google.com/appengine/docs/locations)
    to serve the app from.
    """
    name: pulumi.Output[str]
    """
    Unique name of the app, usually `apps/{PROJECT_ID}`
    """
    project: pulumi.Output[str]
    """
    The project ID to create the application under.
    ~>**NOTE**: GCP only accepts project ID, not project number. If you are using number,
    you may get a "Permission denied" error.
    """
    serving_status: pulumi.Output[str]
    """
    The serving status of the app.
    """
    url_dispatch_rules: pulumi.Output[list]
    """
    A list of dispatch rule blocks. Each block has a `domain`, `path`, and `service` field.
    
      * `domain` (`str`)
      * `path` (`str`)
      * `service` (`str`)
    """
    def __init__(__self__, resource_name, opts=None, auth_domain=None, feature_settings=None, location_id=None, project=None, serving_status=None, __props__=None, __name__=None, __opts__=None):
        """
        Allows creation and management of an App Engine application.
        
        > App Engine applications cannot be deleted once they're created; you have to delete the
           entire project to delete the application. This provider will report the application has been
           successfully deleted; this is a limitation of this provider, and will go away in the future.
           This provider is not able to delete App Engine applications.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] auth_domain: The domain to authenticate users with when using App Engine's User API.
        :param pulumi.Input[dict] feature_settings: A block of optional settings to configure specific App Engine features:
        :param pulumi.Input[str] location_id: The [location](https://cloud.google.com/appengine/docs/locations)
               to serve the app from.
        :param pulumi.Input[str] project: The project ID to create the application under.
               ~>**NOTE**: GCP only accepts project ID, not project number. If you are using number,
               you may get a "Permission denied" error.
        :param pulumi.Input[str] serving_status: The serving status of the app.
        
        The **feature_settings** object supports the following:
        
          * `splitHealthChecks` (`pulumi.Input[bool]`) - Set to false to use the legacy health check instead of the readiness
            and liveness checks.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/app_engine_application.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['auth_domain'] = auth_domain
            __props__['feature_settings'] = feature_settings
            if location_id is None:
                raise TypeError("Missing required property 'location_id'")
            __props__['location_id'] = location_id
            __props__['project'] = project
            __props__['serving_status'] = serving_status
            __props__['app_id'] = None
            __props__['code_bucket'] = None
            __props__['default_bucket'] = None
            __props__['default_hostname'] = None
            __props__['gcr_domain'] = None
            __props__['name'] = None
            __props__['url_dispatch_rules'] = None
        super(Application, __self__).__init__(
            'gcp:appengine/application:Application',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, app_id=None, auth_domain=None, code_bucket=None, default_bucket=None, default_hostname=None, feature_settings=None, gcr_domain=None, location_id=None, name=None, project=None, serving_status=None, url_dispatch_rules=None):
        """
        Get an existing Application resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] app_id: Identifier of the app, usually `{PROJECT_ID}`
        :param pulumi.Input[str] auth_domain: The domain to authenticate users with when using App Engine's User API.
        :param pulumi.Input[str] code_bucket: The GCS bucket code is being stored in for this app.
        :param pulumi.Input[str] default_bucket: The GCS bucket content is being stored in for this app.
        :param pulumi.Input[str] default_hostname: The default hostname for this app.
        :param pulumi.Input[dict] feature_settings: A block of optional settings to configure specific App Engine features:
        :param pulumi.Input[str] gcr_domain: The GCR domain used for storing managed Docker images for this app.
        :param pulumi.Input[str] location_id: The [location](https://cloud.google.com/appengine/docs/locations)
               to serve the app from.
        :param pulumi.Input[str] name: Unique name of the app, usually `apps/{PROJECT_ID}`
        :param pulumi.Input[str] project: The project ID to create the application under.
               ~>**NOTE**: GCP only accepts project ID, not project number. If you are using number,
               you may get a "Permission denied" error.
        :param pulumi.Input[str] serving_status: The serving status of the app.
        :param pulumi.Input[list] url_dispatch_rules: A list of dispatch rule blocks. Each block has a `domain`, `path`, and `service` field.
        
        The **feature_settings** object supports the following:
        
          * `splitHealthChecks` (`pulumi.Input[bool]`) - Set to false to use the legacy health check instead of the readiness
            and liveness checks.
        
        The **url_dispatch_rules** object supports the following:
        
          * `domain` (`pulumi.Input[str]`)
          * `path` (`pulumi.Input[str]`)
          * `service` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/app_engine_application.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["app_id"] = app_id
        __props__["auth_domain"] = auth_domain
        __props__["code_bucket"] = code_bucket
        __props__["default_bucket"] = default_bucket
        __props__["default_hostname"] = default_hostname
        __props__["feature_settings"] = feature_settings
        __props__["gcr_domain"] = gcr_domain
        __props__["location_id"] = location_id
        __props__["name"] = name
        __props__["project"] = project
        __props__["serving_status"] = serving_status
        __props__["url_dispatch_rules"] = url_dispatch_rules
        return Application(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetClientConfigResult:
    """
    A collection of values returned by getClientConfig.
    """
    def __init__(__self__, access_token=None, project=None, region=None, zone=None, id=None):
        if access_token and not isinstance(access_token, str):
            raise TypeError("Expected argument 'access_token' to be a str")
        __self__.access_token = access_token
        """
        The OAuth2 access token used by the client to authenticate against the Google Cloud API.
        """
        if project and not isinstance(project, str):
            raise TypeError("Expected argument 'project' to be a str")
        __self__.project = project
        """
        The ID of the project to apply any resources to.
        """
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        __self__.region = region
        """
        The region to operate under.
        """
        if zone and not isinstance(zone, str):
            raise TypeError("Expected argument 'zone' to be a str")
        __self__.zone = zone
        """
        The zone to operate under.
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetClientConfigResult(GetClientConfigResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetClientConfigResult(
            access_token=self.access_token,
            project=self.project,
            region=self.region,
            zone=self.zone,
            id=self.id)

def get_client_config(opts=None):
    """
    Use this data source to access the configuration of the Google Cloud provider.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/d/client_config.html.markdown.
    """
    __args__ = dict()

    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('gcp:organizations/getClientConfig:getClientConfig', __args__, opts=opts).value

    return AwaitableGetClientConfigResult(
        access_token=__ret__.get('accessToken'),
        project=__ret__.get('project'),
        region=__ret__.get('region'),
        zone=__ret__.get('zone'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Job(pulumi.CustomResource):
    app_engine_http_target: pulumi.Output[dict]
    description: pulumi.Output[str]
    http_target: pulumi.Output[dict]
    name: pulumi.Output[str]
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    pubsub_target: pulumi.Output[dict]
    region: pulumi.Output[str]
    retry_config: pulumi.Output[dict]
    schedule: pulumi.Output[str]
    time_zone: pulumi.Output[str]
    def __init__(__self__, resource_name, opts=None, app_engine_http_target=None, description=None, http_target=None, name=None, project=None, pubsub_target=None, region=None, retry_config=None, schedule=None, time_zone=None, __props__=None, __name__=None, __opts__=None):
        """
        Create a Job resource with the given unique name, props, and options.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        
        The **app_engine_http_target** object supports the following:
        
          * `appEngineRouting` (`pulumi.Input[dict]`)
        
            * `instance` (`pulumi.Input[str]`)
            * `service` (`pulumi.Input[str]`)
            * `version` (`pulumi.Input[str]`)
        
          * `body` (`pulumi.Input[str]`)
          * `headers` (`pulumi.Input[dict]`)
          * `httpMethod` (`pulumi.Input[str]`)
          * `relativeUri` (`pulumi.Input[str]`)
        
        The **http_target** object supports the following:
        
          * `body` (`pulumi.Input[str]`)
          * `headers` (`pulumi.Input[dict]`)
          * `httpMethod` (`pulumi.Input[str]`)
          * `oauthToken` (`pulumi.Input[dict]`)
        
            * `scope` (`pulumi.Input[str]`)
            * `service_account_email` (`pulumi.Input[str]`)
        
          * `oidcToken` (`pulumi.Input[dict]`)
        
            * `audience` (`pulumi.Input[str]`)
            * `service_account_email` (`pulumi.Input[str]`)
        
          * `uri` (`pulumi.Input[str]`)
        
        The **pubsub_target** object supports the following:
        
          * `attributes` (`pulumi.Input[dict]`)
          * `data` (`pulumi.Input[str]`)
          * `topicName` (`pulumi.Input[str]`)
        
        The **retry_config** object supports the following:
        
          * `maxBackoffDuration` (`pulumi.Input[str]`)
          * `maxDoublings` (`pulumi.Input[float]`)
          * `maxRetryDuration` (`pulumi.Input[str]`)
          * `minBackoffDuration` (`pulumi.Input[str]`)
          * `retryCount` (`pulumi.Input[float]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/cloud_scheduler_job.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['app_engine_http_target'] = app_engine_http_target
            __props__['description'] = description
            __props__['http_target'] = http_target
            __props__['name'] = name
            __props__['project'] = project
            __props__['pubsub_target'] = pubsub_target
            __props__['region'] = region
            __props__['retry_config'] = retry_config
            __props__['schedule'] = schedule
            __props__['time_zone'] = time_zone
        super(Job, __self__).__init__(
            'gcp:cloudscheduler/job:Job',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, app_engine_http_target=None, description=None, http_target=None, name=None, project=None, pubsub_target=None, region=None, retry_config=None, schedule=None, time_zone=None):
        """
        Get an existing Job resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        
        The **app_engine_http_target** object supports the following:
        
          * `appEngineRouting` (`pulumi.Input[dict]`)
        
            * `instance` (`pulumi.Input[str]`)
            * `service` (`pulumi.Input[str]`)
            * `version` (`pulumi.Input[str]`)
        
          * `body` (`pulumi.Input[str]`)
          * `headers` (`pulumi.Input[dict]`)
          * `httpMethod` (`pulumi.Input[str]`)
          * `relativeUri` (`pulumi.Input[str]`)
        
        The **http_target** object supports the following:
        
          * `body` (`pulumi.Input[str]`)
          * `headers` (`pulumi.Input[dict]`)
          * `httpMethod` (`pulumi.Input[str]`)
          * `oauthToken` (`pulumi.Input[dict]`)
        
            * `scope` (`pulumi.Input[str]`)
            * `service_account_email` (`pulumi.Input[str]`)
        
          * `oidcToken` (`pulumi.Input[dict]`)
        
            * `audience` (`pulumi.Input[str]`)
            * `service_account_email` (`pulumi.Input[str]`)
        
          * `uri` (`pulumi.Input[str]`)
        
        The **pubsub_target** object supports the following:
        
          * `attributes` (`pulumi.Input[dict]`)
          * `data` (`pulumi.Input[str]`)
          * `topicName` (`pulumi.Input[str]`)
        
        The **retry_config** object supports the following:
        
          * `maxBackoffDuration` (`pulumi.Input[str]`)
          * `maxDoublings` (`pulumi.Input[float]`)
          * `maxRetryDuration` (`pulumi.Input[str]`)
          * `minBackoffDuration` (`pulumi.Input[str]`)
          * `retryCount` (`pulumi.Input[float]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/cloud_scheduler_job.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["app_engine_http_target"] = app_engine_http_target
        __props__["description"] = description
        __props__["http_target"] = http_target
        __props__["name"] = name
        __props__["project"] = project
        __props__["pubsub_target"] = pubsub_target
        __props__["region"] = region
        __props__["retry_config"] = retry_config
        __props__["schedule"] = schedule
        __props__["time_zone"] = time_zone
        return Job(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Service(pulumi.CustomResource):
    apis: pulumi.Output[list]
    config_id: pulumi.Output[str]
    dns_address: pulumi.Output[str]
    endpoints: pulumi.Output[list]
    grpc_config: pulumi.Output[str]
    openapi_config: pulumi.Output[str]
    project: pulumi.Output[str]
    protoc_output_base64: pulumi.Output[str]
    service_name: pulumi.Output[str]
    def __init__(__self__, resource_name, opts=None, grpc_config=None, openapi_config=None, project=None, protoc_output_base64=None, service_name=None, __props__=None, __name__=None, __opts__=None):
        """
        This resource creates and rolls out a Cloud Endpoints service using OpenAPI or gRPC.  View the relevant docs for [OpenAPI](https://cloud.google.com/endpoints/docs/openapi/) and [gRPC](https://cloud.google.com/endpoints/docs/grpc/).
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/endpoints_service.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['grpc_config'] = grpc_config
            __props__['openapi_config'] = openapi_config
            __props__['project'] = project
            __props__['protoc_output_base64'] = protoc_output_base64
            if service_name is None:
                raise TypeError("Missing required property 'service_name'")
            __props__['service_name'] = service_name
            __props__['apis'] = None
            __props__['config_id'] = None
            __props__['dns_address'] = None
            __props__['endpoints'] = None
        super(Service, __self__).__init__(
            'gcp:endpoints/service:Service',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, apis=None, config_id=None, dns_address=None, endpoints=None, grpc_config=None, openapi_config=None, project=None, protoc_output_base64=None, service_name=None):
        """
        Get an existing Service resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        
        The **apis** object supports the following:
        
          * `methods` (`pulumi.Input[list]`)
        
            * `name` (`pulumi.Input[str]`)
            * `requestType` (`pulumi.Input[str]`)
            * `responseType` (`pulumi.Input[str]`)
            * `syntax` (`pulumi.Input[str]`)
        
          * `name` (`pulumi.Input[str]`)
          * `syntax` (`pulumi.Input[str]`)
          * `version` (`pulumi.Input[str]`)
        
        The **endpoints** object supports the following:
        
          * `address` (`pulumi.Input[str]`)
          * `name` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/endpoints_service.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["apis"] = apis
        __props__["config_id"] = config_id
        __props__["dns_address"] = dns_address
        __props__["endpoints"] = endpoints
        __props__["grpc_config"] = grpc_config
        __props__["openapi_config"] = openapi_config
        __props__["project"] = project
        __props__["protoc_output_base64"] = protoc_output_base64
        __props__["service_name"] = service_name
        return Service(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class RouterPeer(pulumi.CustomResource):
    advertise_mode: pulumi.Output[str]
    """
    User-specified flag to indicate which mode to use for advertisement.
    Options include `DEFAULT` or `CUSTOM`.
    """
    advertised_groups: pulumi.Output[list]
    """
    User-specified list of prefix groups to advertise in custom mode,
    which can take one of the following options:
    """
    advertised_ip_ranges: pulumi.Output[list]
    """
    User-specified list of individual IP ranges to advertise in
    custom mode. This field can only be populated if `advertise_mode` is `CUSTOM` and overrides
    the list defined for the router (in the "bgp" message). These IP ranges are advertised in
    addition to any specified groups. Leave this field blank to advertise no custom IP ranges.
    
      * `description` (`str`)
      * `range` (`str`)
    """
    advertised_route_priority: pulumi.Output[float]
    """
    The priority of routes advertised to this BGP peer.
    Changing this forces a new peer to be created.
    """
    interface: pulumi.Output[str]
    """
    The name of the interface the BGP peer is associated with.
    Changing this forces a new peer to be created.
    """
    ip_address: pulumi.Output[str]
    """
    IP address of the interface inside Google Cloud Platform.
    """
    name: pulumi.Output[str]
    """
    A unique name for BGP peer, required by GCE. Changing
    this forces a new peer to be created.
    """
    peer_asn: pulumi.Output[float]
    """
    Peer BGP Autonomous System Number (ASN).
    Changing this forces a new peer to be created.
    """
    peer_ip_address: pulumi.Output[str]
    """
    IP address of the BGP interface outside Google Cloud.
    Changing this forces a new peer to be created.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which this peer's router belongs. If it
    is not provided, the provider project is used. Changing this forces a new peer to be created.
    """
    region: pulumi.Output[str]
    """
    The region this peer's router sits in. If not specified,
    the project region will be used. Changing this forces a new peer to be
    created.
    """
    router: pulumi.Output[str]
    """
    The name of the router in which this BGP peer will be configured.
    Changing this forces a new peer to be created.
    """
    def __init__(__self__, resource_name, opts=None, advertise_mode=None, advertised_groups=None, advertised_ip_ranges=None, advertised_route_priority=None, interface=None, name=None, peer_asn=None, peer_ip_address=None, project=None, region=None, router=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a Cloud Router BGP peer. For more information see
        [the official documentation](https://cloud.google.com/compute/docs/cloudrouter)
        and
        [API](https://cloud.google.com/compute/docs/reference/latest/routers).
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] advertise_mode: User-specified flag to indicate which mode to use for advertisement.
               Options include `DEFAULT` or `CUSTOM`.
        :param pulumi.Input[list] advertised_groups: User-specified list of prefix groups to advertise in custom mode,
               which can take one of the following options:
        :param pulumi.Input[list] advertised_ip_ranges: User-specified list of individual IP ranges to advertise in
               custom mode. This field can only be populated if `advertise_mode` is `CUSTOM` and overrides
               the list defined for the router (in the "bgp" message). These IP ranges are advertised in
               addition to any specified groups. Leave this field blank to advertise no custom IP ranges.
        :param pulumi.Input[float] advertised_route_priority: The priority of routes advertised to this BGP peer.
               Changing this forces a new peer to be created.
        :param pulumi.Input[str] interface: The name of the interface the BGP peer is associated with.
               Changing this forces a new peer to be created.
        :param pulumi.Input[str] name: A unique name for BGP peer, required by GCE. Changing
               this forces a new peer to be created.
        :param pulumi.Input[float] peer_asn: Peer BGP Autonomous System Number (ASN).
               Changing this forces a new peer to be created.
        :param pulumi.Input[str] peer_ip_address: IP address of the BGP interface outside Google Cloud.
               Changing this forces a new peer to be created.
        :param pulumi.Input[str] project: The ID of the project in which this peer's router belongs. If it
               is not provided, the provider project is used. Changing this forces a new peer to be created.
        :param pulumi.Input[str] region: The region this peer's router sits in. If not specified,
               the project region will be used. Changing this forces a new peer to be
               created.
        :param pulumi.Input[str] router: The name of the router in which this BGP peer will be configured.
               Changing this forces a new peer to be created.
        
        The **advertised_ip_ranges** object supports the following:
        
          * `description` (`pulumi.Input[str]`)
          * `range` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/compute_router_peer.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['advertise_mode'] = advertise_mode
            __props__['advertised_groups'] = advertised_groups
            __props__['advertised_ip_ranges'] = advertised_ip_ranges
            __props__['advertised_route_priority'] = advertised_route_priority
            if interface is None:
                raise TypeError("Missing required property 'interface'")
            __props__['interface'] = interface
            __props__['name'] = name
            if peer_asn is None:
                raise TypeError("Missing required property 'peer_asn'")
            __props__['peer_asn'] = peer_asn
            __props__['peer_ip_address'] = peer_ip_address
            __props__['project'] = project
            __props__['region'] = region
            if router is None:
                raise TypeError("Missing required property 'router'")
            __props__['router'] = router
            __props__['ip_address'] = None
        super(RouterPeer, __self__).__init__(
            'gcp:compute/routerPeer:RouterPeer',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, advertise_mode=None, advertised_groups=None, advertised_ip_ranges=None, advertised_route_priority=None, interface=None, ip_address=None, name=None, peer_asn=None, peer_ip_address=None, project=None, region=None, router=None):
        """
        Get an existing RouterPeer resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] advertise_mode: User-specified flag to indicate which mode to use for advertisement.
               Options include `DEFAULT` or `CUSTOM`.
        :param pulumi.Input[list] advertised_groups: User-specified list of prefix groups to advertise in custom mode,
               which can take one of the following options:
        :param pulumi.Input[list] advertised_ip_ranges: User-specified list of individual IP ranges to advertise in
               custom mode. This field can only be populated if `advertise_mode` is `CUSTOM` and overrides
               the list defined for the router (in the "bgp" message). These IP ranges are advertised in
               addition to any specified groups. Leave this field blank to advertise no custom IP ranges.
        :param pulumi.Input[float] advertised_route_priority: The priority of routes advertised to this BGP peer.
               Changing this forces a new peer to be created.
        :param pulumi.Input[str] interface: The name of the interface the BGP peer is associated with.
               Changing this forces a new peer to be created.
        :param pulumi.Input[str] ip_address: IP address of the interface inside Google Cloud Platform.
        :param pulumi.Input[str] name: A unique name for BGP peer, required by GCE. Changing
               this forces a new peer to be created.
        :param pulumi.Input[float] peer_asn: Peer BGP Autonomous System Number (ASN).
               Changing this forces a new peer to be created.
        :param pulumi.Input[str] peer_ip_address: IP address of the BGP interface outside Google Cloud.
               Changing this forces a new peer to be created.
        :param pulumi.Input[str] project: The ID of the project in which this peer's router belongs. If it
               is not provided, the provider project is used. Changing this forces a new peer to be created.
        :param pulumi.Input[str] region: The region this peer's router sits in. If not specified,
               the project region will be used. Changing this forces a new peer to be
               created.
        :param pulumi.Input[str] router: The name of the router in which this BGP peer will be configured.
               Changing this forces a new peer to be created.
        
        The **advertised_ip_ranges** object supports the following:
        
          * `description` (`pulumi.Input[str]`)
          * `range` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/compute_router_peer.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["advertise_mode"] = advertise_mode
        __props__["advertised_groups"] = advertised_groups
        __props__["advertised_ip_ranges"] = advertised_ip_ranges
        __props__["advertised_route_priority"] = advertised_route_priority
        __props__["interface"] = interface
        __props__["ip_address"] = ip_address
        __props__["name"] = name
        __props__["peer_asn"] = peer_asn
        __props__["peer_ip_address"] = peer_ip_address
        __props__["project"] = project
        __props__["region"] = region
        __props__["router"] = router
        return RouterPeer(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Job(pulumi.CustomResource):
    driver_controls_files_uri: pulumi.Output[str]
    """
    If present, the location of miscellaneous control files which may be used as part of job setup and handling. If not present, control files may be placed in the same location as driver_output_uri.
    """
    driver_output_resource_uri: pulumi.Output[str]
    """
    A URI pointing to the location of the stdout of the job's driver program.
    """
    force_delete: pulumi.Output[bool]
    """
    By default, you can only delete inactive jobs within
    Dataproc. Setting this to true, and calling destroy, will ensure that the
    job is first cancelled before issuing the delete.
    """
    hadoop_config: pulumi.Output[dict]
    hive_config: pulumi.Output[dict]
    labels: pulumi.Output[dict]
    """
    The list of labels (key/value pairs) to add to the job.
    """
    pig_config: pulumi.Output[dict]
    placement: pulumi.Output[dict]
    project: pulumi.Output[str]
    """
    The project in which the `cluster` can be found and jobs
    subsequently run against. If it is not provided, the provider project is used.
    """
    pyspark_config: pulumi.Output[dict]
    reference: pulumi.Output[dict]
    region: pulumi.Output[str]
    """
    The Cloud Dataproc region. This essentially determines which clusters are available
    for this job to be submitted to. If not specified, defaults to `global`.
    """
    scheduling: pulumi.Output[dict]
    spark_config: pulumi.Output[dict]
    sparksql_config: pulumi.Output[dict]
    status: pulumi.Output[dict]
    def __init__(__self__, resource_name, opts=None, force_delete=None, hadoop_config=None, hive_config=None, labels=None, pig_config=None, placement=None, project=None, pyspark_config=None, reference=None, region=None, scheduling=None, spark_config=None, sparksql_config=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a job resource within a Dataproc cluster within GCE. For more information see
        [the official dataproc documentation](https://cloud.google.com/dataproc/).
        
        !> **Note:** This resource does not support 'update' and changing any attributes will cause the resource to be recreated.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] force_delete: By default, you can only delete inactive jobs within
               Dataproc. Setting this to true, and calling destroy, will ensure that the
               job is first cancelled before issuing the delete.
        :param pulumi.Input[dict] labels: The list of labels (key/value pairs) to add to the job.
        :param pulumi.Input[str] project: The project in which the `cluster` can be found and jobs
               subsequently run against. If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Cloud Dataproc region. This essentially determines which clusters are available
               for this job to be submitted to. If not specified, defaults to `global`.
        
        The **hadoop_config** object supports the following:
        
          * `archiveUris` (`pulumi.Input[list]`)
          * `args` (`pulumi.Input[list]`)
          * `fileUris` (`pulumi.Input[list]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `mainClass` (`pulumi.Input[str]`)
          * `mainJarFileUri` (`pulumi.Input[str]`)
          * `properties` (`pulumi.Input[dict]`)
        
        The **hive_config** object supports the following:
        
          * `continueOnFailure` (`pulumi.Input[bool]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `properties` (`pulumi.Input[dict]`)
          * `queryFileUri` (`pulumi.Input[str]`)
          * `queryLists` (`pulumi.Input[list]`)
          * `scriptVariables` (`pulumi.Input[dict]`)
        
        The **pig_config** object supports the following:
        
          * `continueOnFailure` (`pulumi.Input[bool]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `properties` (`pulumi.Input[dict]`)
          * `queryFileUri` (`pulumi.Input[str]`)
          * `queryLists` (`pulumi.Input[list]`)
          * `scriptVariables` (`pulumi.Input[dict]`)
        
        The **placement** object supports the following:
        
          * `clusterName` (`pulumi.Input[str]`)
          * `clusterUuid` (`pulumi.Input[str]`)
        
        The **pyspark_config** object supports the following:
        
          * `archiveUris` (`pulumi.Input[list]`)
          * `args` (`pulumi.Input[list]`)
          * `fileUris` (`pulumi.Input[list]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `mainPythonFileUri` (`pulumi.Input[str]`)
          * `properties` (`pulumi.Input[dict]`)
          * `pythonFileUris` (`pulumi.Input[list]`)
        
        The **reference** object supports the following:
        
          * `job_id` (`pulumi.Input[str]`)
        
        The **scheduling** object supports the following:
        
          * `maxFailuresPerHour` (`pulumi.Input[float]`)
        
        The **spark_config** object supports the following:
        
          * `archiveUris` (`pulumi.Input[list]`)
          * `args` (`pulumi.Input[list]`)
          * `fileUris` (`pulumi.Input[list]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `mainClass` (`pulumi.Input[str]`)
          * `mainJarFileUri` (`pulumi.Input[str]`)
          * `properties` (`pulumi.Input[dict]`)
        
        The **sparksql_config** object supports the following:
        
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `properties` (`pulumi.Input[dict]`)
          * `queryFileUri` (`pulumi.Input[str]`)
          * `queryLists` (`pulumi.Input[list]`)
          * `scriptVariables` (`pulumi.Input[dict]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/dataproc_job.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['force_delete'] = force_delete
            __props__['hadoop_config'] = hadoop_config
            __props__['hive_config'] = hive_config
            __props__['labels'] = labels
            __props__['pig_config'] = pig_config
            if placement is None:
                raise TypeError("Missing required property 'placement'")
            __props__['placement'] = placement
            __props__['project'] = project
            __props__['pyspark_config'] = pyspark_config
            __props__['reference'] = reference
            __props__['region'] = region
            __props__['scheduling'] = scheduling
            __props__['spark_config'] = spark_config
            __props__['sparksql_config'] = sparksql_config
            __props__['driver_controls_files_uri'] = None
            __props__['driver_output_resource_uri'] = None
            __props__['status'] = None
        super(Job, __self__).__init__(
            'gcp:dataproc/job:Job',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, driver_controls_files_uri=None, driver_output_resource_uri=None, force_delete=None, hadoop_config=None, hive_config=None, labels=None, pig_config=None, placement=None, project=None, pyspark_config=None, reference=None, region=None, scheduling=None, spark_config=None, sparksql_config=None, status=None):
        """
        Get an existing Job resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] driver_controls_files_uri: If present, the location of miscellaneous control files which may be used as part of job setup and handling. If not present, control files may be placed in the same location as driver_output_uri.
        :param pulumi.Input[str] driver_output_resource_uri: A URI pointing to the location of the stdout of the job's driver program.
        :param pulumi.Input[bool] force_delete: By default, you can only delete inactive jobs within
               Dataproc. Setting this to true, and calling destroy, will ensure that the
               job is first cancelled before issuing the delete.
        :param pulumi.Input[dict] labels: The list of labels (key/value pairs) to add to the job.
        :param pulumi.Input[str] project: The project in which the `cluster` can be found and jobs
               subsequently run against. If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Cloud Dataproc region. This essentially determines which clusters are available
               for this job to be submitted to. If not specified, defaults to `global`.
        
        The **hadoop_config** object supports the following:
        
          * `archiveUris` (`pulumi.Input[list]`)
          * `args` (`pulumi.Input[list]`)
          * `fileUris` (`pulumi.Input[list]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `mainClass` (`pulumi.Input[str]`)
          * `mainJarFileUri` (`pulumi.Input[str]`)
          * `properties` (`pulumi.Input[dict]`)
        
        The **hive_config** object supports the following:
        
          * `continueOnFailure` (`pulumi.Input[bool]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `properties` (`pulumi.Input[dict]`)
          * `queryFileUri` (`pulumi.Input[str]`)
          * `queryLists` (`pulumi.Input[list]`)
          * `scriptVariables` (`pulumi.Input[dict]`)
        
        The **pig_config** object supports the following:
        
          * `continueOnFailure` (`pulumi.Input[bool]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `properties` (`pulumi.Input[dict]`)
          * `queryFileUri` (`pulumi.Input[str]`)
          * `queryLists` (`pulumi.Input[list]`)
          * `scriptVariables` (`pulumi.Input[dict]`)
        
        The **placement** object supports the following:
        
          * `clusterName` (`pulumi.Input[str]`)
          * `clusterUuid` (`pulumi.Input[str]`)
        
        The **pyspark_config** object supports the following:
        
          * `archiveUris` (`pulumi.Input[list]`)
          * `args` (`pulumi.Input[list]`)
          * `fileUris` (`pulumi.Input[list]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `mainPythonFileUri` (`pulumi.Input[str]`)
          * `properties` (`pulumi.Input[dict]`)
          * `pythonFileUris` (`pulumi.Input[list]`)
        
        The **reference** object supports the following:
        
          * `job_id` (`pulumi.Input[str]`)
        
        The **scheduling** object supports the following:
        
          * `maxFailuresPerHour` (`pulumi.Input[float]`)
        
        The **spark_config** object supports the following:
        
          * `archiveUris` (`pulumi.Input[list]`)
          * `args` (`pulumi.Input[list]`)
          * `fileUris` (`pulumi.Input[list]`)
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `mainClass` (`pulumi.Input[str]`)
          * `mainJarFileUri` (`pulumi.Input[str]`)
          * `properties` (`pulumi.Input[dict]`)
        
        The **sparksql_config** object supports the following:
        
          * `jarFileUris` (`pulumi.Input[list]`)
          * `loggingConfig` (`pulumi.Input[dict]`)
        
            * `driverLogLevels` (`pulumi.Input[dict]`)
        
          * `properties` (`pulumi.Input[dict]`)
          * `queryFileUri` (`pulumi.Input[str]`)
          * `queryLists` (`pulumi.Input[list]`)
          * `scriptVariables` (`pulumi.Input[dict]`)
        
        The **status** object supports the following:
        
          * `details` (`pulumi.Input[str]`)
          * `state` (`pulumi.Input[str]`)
          * `stateStartTime` (`pulumi.Input[str]`)
          * `substate` (`pulumi.Input[str]`)

        > This content is derived from https://github.com/terraform-providers/terraform-provider-google/blob/master/website/docs/r/dataproc_job.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["driver_controls_files_uri"] = driver_controls_files_uri
        __props__["driver_output_resource_uri"] = driver_output_resource_uri
        __props__["force_delete"] = force_delete
        __props__["hadoop_config"] = hadoop_config
        __props__["hive_config"] = hive_config
        __props__["labels"] = labels
        __props__["pig_config"] = pig_config
        __props__["placement"] = placement
        __props__["project"] = project
        __props__["pyspark_config"] = pyspark_config
        __props__["reference"] = reference
        __props__["region"] = region
        __props__["scheduling"] = scheduling
        __props__["spark_config"] = spark_config
        __props__["sparksql_config"] = sparksql_config
        __props__["status"] = status
        return Job(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

